@@ -99,7 +99,7 @@
  * failure.
  */
 
-static gboolean continue_after_wtap_open_offline_failure = TRUE;
+static gboolean stop_after_wtap_open_offline_failure = FALSE;
 
 /*
  * table report variables
@@ -1574,7 +1574,7 @@ real_main(int argc, char *argv[])
         break;
 
       case 'C':
-        continue_after_wtap_open_offline_failure = FALSE;
+        stop_after_wtap_open_offline_failure = TRUE;
         break;
 
       case 'A':
@@ -1699,7 +1699,7 @@ real_main(int argc, char *argv[])
     if (!wth) {
       cfile_open_failure_message("capinfos", argv[opt], err, err_info);
       overall_error_status = 2; /* remember that an error has occurred */
-      if (!continue_after_wtap_open_offline_failure)
+      if (stop_after_wtap_open_offline_failure)
         goto exit;
     }
 
