@@ -5,19 +5,7 @@
  * By Gerald Combs <gerald@wireshark.org>
  * Copyright 1998 Gerald Combs
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ * SPDX-License-Identifier: GPL-2.0-or-later
  */
 
 #include "config.h"
@@ -1374,15 +1362,15 @@ guint8 *
 get_t61_string(wmem_allocator_t *scope, const guint8 *ptr, gint length)
 {
     gint           i;
-    guint8        *c;
+    const guint8  *c;
     wmem_strbuf_t *strbuf;
 
     strbuf = wmem_strbuf_sized_new(scope, length+1, 0);
 
-    for (i = 0, c = (guint8 *)ptr; i < length; c++, i++) {
+    for (i = 0, c = ptr; i < length; c++, i++) {
         if (!t61_tab[*c]) {
             wmem_strbuf_append_unichar(strbuf, UNREPL);
-        } else if ((*c & 0xf0) == 0xc0) {
+        } else if (i < length - 1 && (*c & 0xf0) == 0xc0) {
             gint j = *c & 0x0f;
             /* If this is the end of the string, or if the base
              * character is just a space, treat this as a regular
