@@ -5,19 +5,7 @@
  * By Gerald Combs <gerald@wireshark.org>
  * Copyright 1998 Gerald Combs
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ * SPDX-License-Identifier: GPL-2.0-or-later
  */
 /*
  * Updated 1 Dec 10 jjm
@@ -173,10 +161,8 @@ color_filters_set_tmp(guint8 filt_nr, const gchar *filter, gboolean disabled, gc
                 g_free(local_err_msg);
                 return FALSE;
             } else {
-                if (colorf->filter_text != NULL)
-                    g_free(colorf->filter_text);
-                if (colorf->c_colorfilter != NULL)
-                    dfilter_free(colorf->c_colorfilter);
+                g_free(colorf->filter_text);
+                dfilter_free(colorf->c_colorfilter);
                 colorf->filter_text = g_strdup(tmpfilter);
                 colorf->c_colorfilter = compiled_filter;
                 colorf->disabled = ((i!=filt_nr) ? TRUE : disabled);
@@ -225,12 +211,9 @@ color_filters_reset_tmp(gchar **err_msg)
 void
 color_filter_delete(color_filter_t *colorf)
 {
-    if (colorf->filter_name != NULL)
-        g_free(colorf->filter_name);
-    if (colorf->filter_text != NULL)
-        g_free(colorf->filter_text);
-    if (colorf->c_colorfilter != NULL)
-        dfilter_free(colorf->c_colorfilter);
+    g_free(colorf->filter_name);
+    g_free(colorf->filter_text);
+    dfilter_free(colorf->c_colorfilter);
     g_free(colorf);
 }
 
@@ -431,12 +414,12 @@ color_filter_validate_cb(gpointer filter_arg, gpointer err)
     if (colorf->disabled) return;
 
     if (!dfilter_compile(colorf->filter_text, &colorf->c_colorfilter, &local_err_msg)) {
-        *err_msg = g_strdup_printf("Removing color filter name: \"%s\" text: \"%s\".\n%s",
+        *err_msg = g_strdup_printf("Disabling color filter name: \"%s\" filter: \"%s\".\n%s",
                       colorf->filter_name, colorf->filter_text, local_err_msg);
         g_free(local_err_msg);
-        /* Delete the color filter from the list of color filters. */
-        color_filter_valid_list = g_slist_remove(color_filter_valid_list, colorf);
-        color_filter_delete(colorf);
+
+        /* Disable the color filter in the list of color filters. */
+        colorf->disabled = TRUE;
     }
 }
 
@@ -555,6 +538,8 @@ read_filters_file(const gchar *path, FILE *f, gpointer user_data, color_filter_a
     name = (gchar *)g_malloc(name_len + 1);
     filter_exp = (gchar *)g_malloc(filter_exp_len + 1);
 
+    prefs.unknown_colorfilters = FALSE;
+
     while (1) {
 
         if (skip_end_of_line) {
@@ -720,6 +705,7 @@ color_filters_read_globals(gpointer user_data, gchar** err_msg, color_filter_add
          * There is no global filter file; treat that as equivalent to
          * that file existing bug being empty, and say we succeeded.
          */
+        g_free(path);
         return TRUE;
     }
 
