@@ -5,19 +5,7 @@
  * By Gerald Combs <gerald@wireshark.org>
  * Copyright 1998 Gerald Combs
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ * SPDX-License-Identifier: GPL-2.0-or-later
  */
 
 #include "config.h"
@@ -524,8 +512,8 @@ proto_register_vdp(void)
 		{ "Data", "vdp21.data", FT_BYTES, BASE_NONE,
 			NULL, 0x0, NULL, HFILL }},
 		{ &hf_vdp_tlv_org_oui,
-		{ "VIDSTR VID", "vdp21.org_oui", FT_UINT24, BASE_HEX,
-			VALS(oui_vals), 0x0, NULL, HFILL }},
+		{ "VIDSTR VID", "vdp21.org_oui", FT_UINT24, BASE_OUI,
+			NULL, 0x0, NULL, HFILL }},
 		{ &hf_vdp_tlv_oracle_subtype,
 		{ "Oracle Subtype", "vdp21.org.oracle.subtype", FT_UINT8, BASE_HEX,
 			VALS(oui_oracle_subtype_vals), 0x0, NULL, HFILL }},
