@@ -6,19 +6,7 @@
  * By Gerald Combs <gerald@wireshark.org>
  * Copyright 1998 Gerald Combs
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ * SPDX-License-Identifier: GPL-2.0-or-later
  */
 
 #include "config.h"
@@ -536,11 +524,11 @@ dissect_fcfzs(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, void* data)
 
     if ((opcode != FCCT_MSG_ACC) && (opcode != FCCT_MSG_RJT)) {
         conversation = find_conversation(pinfo->num, &pinfo->src, &pinfo->dst,
-                                         pinfo->ptype, fchdr->oxid,
+                                         conversation_pt_to_endpoint_type(pinfo->ptype), fchdr->oxid,
                                          fchdr->rxid, NO_PORT2);
         if (!conversation) {
             conversation = conversation_new(pinfo->num, &pinfo->src, &pinfo->dst,
-                                            pinfo->ptype, fchdr->oxid,
+                                            conversation_pt_to_endpoint_type(pinfo->ptype), fchdr->oxid,
                                             fchdr->rxid, NO_PORT2);
         }
 
@@ -571,7 +559,7 @@ dissect_fcfzs(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, void* data)
     else {
         /* Opcode is ACC or RJT */
         conversation = find_conversation(pinfo->num, &pinfo->src, &pinfo->dst,
-                                         pinfo->ptype, fchdr->oxid,
+                                         conversation_pt_to_endpoint_type(pinfo->ptype), fchdr->oxid,
                                          fchdr->rxid, NO_PORT2);
         isreq = FALSE;
         if (!conversation) {
