@@ -13,23 +13,16 @@
  *                 IEC 61883-6 audio/music data transmission protocol improved.
  *                 Changes to meet 1722 Draft 15 specification.
  *
+ * Copyright 2017, Marouen Ghodhbane <marouen.ghodhbane@nxp.com>
+ *                 Dissection for the 1722 Compressed Video subtype added.
+ *                 CVF Format subtype supported: H264 and MJPEG
+ *                 The dissection meets the 1722-2016 specification.
+ *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
  * Copyright 1998 Gerald Combs
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ * SPDX-License-Identifier: GPL-2.0-or-later
  *
  * The 1722 Protocol specification can be found at the following:
  * http://grouper.ieee.org/groups/1722/
@@ -50,6 +43,8 @@ void proto_register_1722_aaf(void);
 void proto_reg_handoff_1722_aaf(void);
 void proto_register_1722_61883(void);
 void proto_reg_handoff_1722_61883(void);
+void proto_register_1722_cvf(void);
+void proto_reg_handoff_1722_cvf(void);
 
 /**************************************************************************************************/
 /* 1722                                                                                           */
@@ -57,6 +52,7 @@ void proto_reg_handoff_1722_61883(void);
 /**************************************************************************************************/
 #define IEEE_1722_SUBTYPE_61883             0x00
 #define IEEE_1722_SUBTYPE_AAF               0x02
+#define IEEE_1722_SUBTYPE_CVF               0x03
 #define IEEE_1722_SUBTYPE_CRF               0x04
 
 /* Bit Field Masks */
@@ -127,6 +123,36 @@ void proto_reg_handoff_1722_61883(void);
 #define IEEE_1722_DATA_MASK                             0x00
 #define IEEE_1722_SAMPLE_MASK                           0x00
 
+/**************************************************************************************************/
+/* subtype CVF                                                                                    */
+/*                                                                                                */
+/**************************************************************************************************/
+#define IEEE_1722_CVF_FORMAT_SUBTYPE_MJPEG              0x00
+#define IEEE_1722_CVF_FORMAT_SUBTYPE_H264               0x01
+#define IEEE_1722_CVF_FORMAT_SUBTYPE_JPEG2000           0x02
+
+/* Bit Field Masks */
+#define IEEE_1722_MR_MASK                               0x08
+#define IEEE_1722_TV_MASK                               0x01
+#define IEEE_1722_SEQ_NUM_MASK                          0x00
+#define IEEE_1722_TU_MASK                               0x01
+#define IEEE_1722_STREAM_ID_MASK                        0x00
+#define IEEE_1722_TIMESTAMP_MASK                        0x00
+#define IEEE_1722_FORMAT_MASK                           0x00
+#define IEEE_1722_FORMAT_SUBTYPE_MASK                   0x00
+#define IEEE_1722_CVF_H264_TIMESTAMP_MASK               0x00
+#define IEEE_1722_CVF_H264_FORBIDDEN_MASK               0x80
+#define IEEE_1722_CVF_H264_NRI_MASK                     0x60
+#define IEEE_1722_CVF_H264_NAL_TYPE_MASK                0x1f
+#define IEEE_1722_CVF_MJPEG_TS_MASK                     0x00
+#define IEEE_1722_CVF_MJPEG_FRAGMENT_OFFSET_MASK        0x00
+#define IEEE_1722_CVF_MJPEG_TYPE_MASK                   0x00
+#define IEEE_1722_CVF_MJPEG_Q_MASK                      0x00
+#define IEEE_1722_CVF_MJPEG_WIDTH_MASK                  0x00
+#define IEEE_1722_CVF_MJPEG_HEIGHT_MASK                 0x00
+#define IEEE_1722_MARKER_BIT_MASK                       0x10
+#define IEEE_1722_EVT_MASK                              0x0f
+
 /**************************************************************************************************/
 /* subtype CRF                                                                                    */
 /*                                                                                                */
@@ -441,6 +467,81 @@ static int ett_1722_crf_timestamp = -1;
 /* Initialize expert fields */
 static expert_field ei_crf_datalen = EI_INIT;
 
+/**************************************************************************************************/
+/* subtype CVF                                                                                    */
+/*                                                                                                */
+/**************************************************************************************************/
+static const range_string cvf_format_range_rvals [] = {
+    {0, 1,      "Reserved"},
+    {2, 2,      "RFC payload type"},
+    {3, 0xff,   "Reserved"},
+    {0, 0,      NULL}
+};
+
+static const true_false_string tfs_marker_bit_set_not_set = { "Marker Bit is set", "Marker bit not set" };
+
+static const range_string cvf_format_subtype_range_rvals [] = {
+    {0, 0,      "MJPEG Format (RFC2435)"},
+    {1, 1,      "H264 Format"},
+    {2, 2,      "JPEG 2000 Video"},
+    {3, 0xff,   "Reserved"},
+    {0, 0,      NULL}
+};
+
+static const range_string cvf_h264_nal_type_range_rvals [] = {
+    {0x00, 0x00,    "Undefined"},
+    {0x01, 0x17,    "Single Nal Unit Packet"},
+    {0x18, 0x18,    "STAP-A"},
+    {0x19, 0x19,    "STAP-B"},
+    {0x1a, 0x1a,    "MTAP16"},
+    {0x1b, 0x1b,    "MTAP24"},
+    {0x1c, 0x1c,    "FU-A"},
+    {0x1d, 0x1d,    "FU-B"},
+    {0x1e, 0xff,    "Unspecified"},
+    {0, 0,      NULL}
+};
+
+static const range_string cvf_mjpeg_ts_range_rvals [] = {
+    {0, 0,          "MJPEG TYPE SPEC PROGRESSIVE"},
+    {1, 1,          "MJPEG TYPE SPEC ODD"},
+    {2, 2,          "MJPEG TYPE SPEC EVEN"},
+    {3, 3,          "MJPEG TYPE SPEC SINGLE"},
+    {3, 0xff,       "Unspecified"},
+    {0, 0,      NULL}
+};
+
+/* Initialize the protocol and registered fields          */
+
+static int proto_1722_cvf = -1;
+static int hf_1722_cvf_mrfield = -1;
+static int hf_1722_cvf_tvfield = -1;
+static int hf_1722_cvf_seqnum = -1;
+static int hf_1722_cvf_tufield = -1;
+static int hf_1722_cvf_stream_id = -1;
+static int hf_1722_cvf_avtp_timestamp = -1;
+static int hf_1722_cvf_format = -1;
+static int hf_1722_cvf_format_subtype = -1;
+static int hf_1722_cvf_stream_data_length = -1;
+static int hf_1722_cvf_evtfield = -1;
+static int hf_1722_cvf_marker_bit = -1;
+static int hf_1722_cvf_h264_timestamp = -1;
+static int hf_1722_cvf_h264_forbidden_bit = -1;
+static int hf_1722_cvf_h264_nri = -1;
+static int hf_1722_cvf_h264_nal_type = -1;
+static int hf_1722_cvf_mjpeg_ts = -1;
+static int hf_1722_cvf_mjpeg_fragment_offset = -1;
+static int hf_1722_cvf_mjpeg_type = -1;
+static int hf_1722_cvf_mjpeg_q = -1;
+static int hf_1722_cvf_mjpeg_width = -1;
+static int hf_1722_cvf_mjpeg_height = -1;
+
+/* Initialize the subtree pointers */
+static int ett_1722_cvf = -1;
+
+/* Initialize expert fields */
+static expert_field ei_cvf_jpeg2000_format = EI_INIT;
+static expert_field ei_cvf_reserved_format = EI_INIT;
+
 /**************************************************************************************************/
 /* 1722 dissector implementation                                                                  */
 /*                                                                                                */
@@ -467,7 +568,7 @@ static int dissect_1722(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, voi
     offset += 1;
     proto_tree_add_bitmask_list(ieee1722_tree, tvb, offset, 1, fields, ENC_NA);
 
-    /* call any registered subtype dissectors which use only the common AVTPDU (e.g. 1722.1, MAAP, 61883, AAF or CRF) */
+    /* call any registered subtype dissectors which use only the common AVTPDU (e.g. 1722.1, MAAP, 61883, AAF, CRF or CVF) */
     if (dissector_try_uint(avb_dissector_table, subtype, tvb, pinfo, tree))
         return tvb_captured_length(tvb);
 
@@ -503,7 +604,7 @@ void proto_register_1722(void)
     proto_register_field_array(proto_1722, hf, array_length(hf));
     proto_register_subtree_array(ett, array_length(ett));
 
-    /* Sub-dissector for 1772.1, 1722 AAF, 1722 CRF, 1722 61883 */
+    /* Sub-dissector for 1772.1, 1722 AAF, 1722 CRF, 1722 61883, 1722 CVF */
     avb_dissector_table = register_dissector_table("ieee1722.subtype",
                           "IEEE1722 AVBTP Subtype", proto_1722, FT_UINT8, BASE_HEX);
 }
@@ -803,127 +904,127 @@ void proto_register_1722_61883(void)
 {
     static hf_register_info hf[] = {
         { &hf_1722_61883_mrfield,
-            { "Media Clock Restart", "61883.mrfield",
+            { "Media Clock Restart", "iec61883.mrfield",
               FT_BOOLEAN, 8, NULL, IEEE_1722_MR_MASK, NULL, HFILL }
         },
         { &hf_1722_61883_gvfield,
-            { "Gateway Info Valid", "61883.gvfield",
+            { "Gateway Info Valid", "iec61883.gvfield",
               FT_BOOLEAN, 8, NULL, IEEE_1722_GV_MASK, NULL, HFILL }
         },
         { &hf_1722_61883_tvfield,
-            { "Timestamp Valid", "61883.tvfield",
+            { "Timestamp Valid", "iec61883.tvfield",
               FT_BOOLEAN, 8, NULL, IEEE_1722_TV_MASK, NULL, HFILL }
             },
         { &hf_1722_61883_seqnum,
-            { "Sequence Number", "61883.seqnum",
+            { "Sequence Number", "iec61883.seqnum",
               FT_UINT8, BASE_HEX, NULL, 0x00, NULL, HFILL }
         },
         { &hf_1722_61883_tufield,
-            { "Timestamp Uncertain", "61883.tufield",
+            { "Timestamp Uncertain", "iec61883.tufield",
               FT_BOOLEAN, 8, NULL, IEEE_1722_TU_MASK, NULL, HFILL }
         },
         { &hf_1722_61883_stream_id,
-            { "Stream ID", "61883.stream_id",
+            { "Stream ID", "iec61883.stream_id",
               FT_UINT64, BASE_HEX, NULL, 0x00, NULL, HFILL }
         },
         { &hf_1722_61883_avtp_timestamp,
-            { "AVTP Timestamp", "61883.avtp_timestamp",
+            { "AVTP Timestamp", "iec61883.avtp_timestamp",
               FT_UINT32, BASE_HEX, NULL, 0x00, NULL, HFILL }
         },
         { &hf_1722_61883_gateway_info,
-            { "Gateway Info", "61883.gateway_info",
+            { "Gateway Info", "iec61883.gateway_info",
               FT_UINT32, BASE_HEX, NULL, 0x00, NULL, HFILL }
         },
         { &hf_1722_61883_stream_data_length,
-            { "1394 Stream Data Length", "61883.stream_data_len",
+            { "1394 Stream Data Length", "iec61883.stream_data_len",
               FT_UINT16, BASE_DEC|BASE_UNIT_STRING, &units_byte_bytes, 0x00, NULL, HFILL }
         },
         { &hf_1722_61883_tag,
-            { "1394 Packet Format Tag", "61883.tag",
+            { "1394 Packet Format Tag", "iec61883.tag",
               FT_UINT8, BASE_HEX, VALS(tag_vals), IEEE_1722_TAG_MASK, NULL, HFILL }
         },
         { &hf_1722_61883_channel,
-            { "1394 Packet Channel", "61883.channel",
+            { "1394 Packet Channel", "iec61883.channel",
                 FT_UINT8, BASE_DEC, NULL, IEEE_1722_CHANNEL_MASK, NULL, HFILL }
         },
         { &hf_1722_61883_tcode,
-            { "1394 Packet Tcode", "61883.tcode",
+            { "1394 Packet Tcode", "iec61883.tcode",
               FT_UINT8, BASE_HEX, NULL, IEEE_1722_TCODE_MASK, NULL, HFILL }
         },
         { &hf_1722_61883_sy,
-            { "1394 App-specific Control", "61883.sy",
+            { "1394 App-specific Control", "iec61883.sy",
               FT_UINT8, BASE_HEX, NULL, IEEE_1722_SY_MASK, NULL, HFILL }
         },
         { &hf_1722_61883_cip_qi1,
-            { "CIP Quadlet Indicator 1", "61883.qi1",
+            { "CIP Quadlet Indicator 1", "iec61883.qi1",
               FT_UINT8, BASE_HEX, NULL, IEEE_1722_QI1_MASK, NULL, HFILL }
         },
         { &hf_1722_61883_cip_sid,
-            { "CIP Source ID", "61883.sid",
+            { "CIP Source ID", "iec61883.sid",
               FT_UINT8, BASE_DEC, NULL, IEEE_1722_SID_MASK, NULL, HFILL }
         },
         { &hf_1722_61883_cip_dbs,
-            { "CIP Data Block Size", "61883.dbs",
+            { "CIP Data Block Size", "iec61883.dbs",
               FT_UINT8, BASE_HEX, NULL, 0x00, NULL, HFILL }
         },
         { &hf_1722_61883_cip_fn,
-            { "CIP Fraction Number", "61883.fn",
+            { "CIP Fraction Number", "iec61883.fn",
               FT_UINT8, BASE_HEX, VALS(fraction_number_vals), IEEE_1722_FN_MASK, NULL, HFILL }
         },
         { &hf_1722_61883_cip_qpc,
-            { "CIP Quadlet Padding Count", "61883.qpc",
+            { "CIP Quadlet Padding Count", "iec61883.qpc",
               FT_UINT8, BASE_HEX, NULL, IEEE_1722_QPC_MASK, NULL, HFILL }
         },
         { &hf_1722_61883_cip_sph,
-            { "CIP Source Packet Header", "61883.sph",
+            { "CIP Source Packet Header", "iec61883.sph",
               FT_BOOLEAN, 8, NULL, IEEE_1722_SPH_MASK, NULL, HFILL }
         },
         { &hf_1722_61883_cip_dbc,
-            { "CIP Data Block Continuity", "61883.dbc",
+            { "CIP Data Block Continuity", "iec61883.dbc",
               FT_UINT8, BASE_HEX, NULL, 0x00, NULL, HFILL }
         },
         { &hf_1722_61883_cip_qi2,
-            { "CIP Quadlet Indicator 2", "61883.qi2",
+            { "CIP Quadlet Indicator 2", "iec61883.qi2",
               FT_UINT8, BASE_HEX, NULL, IEEE_1722_QI2_MASK, NULL, HFILL }
         },
         { &hf_1722_61883_cip_fmt,
-            { "CIP Format ID", "61883.fmt",
+            { "CIP Format ID", "iec61883.fmt",
               FT_UINT8, BASE_HEX | BASE_RANGE_STRING, RVALS(format_rvals), IEEE_1722_FMT_MASK, NULL, HFILL }
         },
         { &hf_1722_61883_cip_fdf_no_syt,
-            { "CIP Format Dependent Field", "61883.fdf_no_syt",
+            { "CIP Format Dependent Field", "iec61883.fdf_no_syt",
               FT_UINT8, BASE_HEX, NULL, 0x00, NULL, HFILL }
         },
         { &hf_1722_61883_cip_fdf_tsf,
-            { "Time shift flag", "61883.fdf_tsf",
+            { "Time shift flag", "iec61883.fdf_tsf",
               FT_BOOLEAN, 8, NULL, IEEE_1722_FDF_TSF_MASK, NULL, HFILL }
         },
         { &hf_1722_61883_cip_fdf,
-            { "CIP Format Dependent Field", "61883.fdf",
+            { "CIP Format Dependent Field", "iec61883.fdf",
               FT_UINT8, BASE_HEX | BASE_RANGE_STRING, RVALS(fdf_rvals), IEEE_1722_FDF_MASK, NULL, HFILL }
         },
         { &hf_1722_61883_cip_syt,
-            { "CIP SYT", "61883.syt",
+            { "CIP SYT", "iec61883.syt",
               FT_UINT16, BASE_HEX | BASE_RANGE_STRING, RVALS(syt_rvals), 0x00, NULL, HFILL }
         },
         { &hf_1722_61883_audio_data,
-            { "Audio Data", "61883.audiodata",
+            { "Audio Data", "iec61883.audiodata",
               FT_BYTES, BASE_NONE, NULL, 0x00, NULL, HFILL }
         },
         { &hf_1722_61883_label,
-            { "Label", "61883.audiodata.sample.label",
+            { "Label", "iec61883.audiodata.sample.label",
               FT_UINT8, BASE_HEX, NULL, 0x00, NULL, HFILL }
         },
         { &hf_1722_61883_sample,
-            { "Sample", "61883.audiodata.sample.sampledata",
+            { "Sample", "iec61883.audiodata.sample.sampledata",
               FT_BYTES, BASE_NONE, NULL, 0x00, NULL, HFILL }
         },
         { &hf_1722_61883_video_data,
-            { "Video Data", "61883.videodata",
+            { "Video Data", "iec61883.videodata",
               FT_BYTES, BASE_NONE, NULL, 0x00, NULL, HFILL }
         },
         { &hf_1722_61883_source_packet_header_timestamp,
-            { "Source Packet Header Timestamp", "61883.spht",
+            { "Source Packet Header Timestamp", "iec61883.spht",
               FT_UINT32, BASE_HEX, NULL, 0x00, NULL, HFILL }
         }
     };
@@ -936,33 +1037,33 @@ void proto_register_1722_61883(void)
     };
 
     static ei_register_info ei[] = {
-        { &ei_1722_61883_incorrect_tag,         { "61883.incorrect_tag", PI_PROTOCOL, PI_WARN,
+        { &ei_1722_61883_incorrect_tag,         { "iec61883.incorrect_tag", PI_PROTOCOL, PI_WARN,
                                                   "Incorrect tag field, only 0x00 and 0x01 supported for AVTP", EXPFILL }},
-        { &ei_1722_61883_incorrect_tcode,       { "61883.incorrect_tcode", PI_PROTOCOL, PI_WARN,
+        { &ei_1722_61883_incorrect_tcode,       { "iec61883.incorrect_tcode", PI_PROTOCOL, PI_WARN,
                                                   "Incorrect tcode, talker shall set this field to 0x0A", EXPFILL }},
-        { &ei_1722_61883_incorrect_qi1,         { "61883.incorrect_qi1", PI_PROTOCOL, PI_WARN,
+        { &ei_1722_61883_incorrect_qi1,         { "iec61883.incorrect_qi1", PI_PROTOCOL, PI_WARN,
                                                   "Incorrect quadlet indicator 1 field, talker shall set this field to 0x00", EXPFILL }},
-        { &ei_1722_61883_incorrect_qpc,         { "61883.incorrect_qpc", PI_PROTOCOL, PI_WARN,
+        { &ei_1722_61883_incorrect_qpc,         { "iec61883.incorrect_qpc", PI_PROTOCOL, PI_WARN,
                                                   "Incorrect quadlet padding count field, shall be set to 0", EXPFILL }},
-        { &ei_1722_61883_incorrect_qi2,         { "61883.incorrect_qi2", PI_PROTOCOL, PI_WARN,
+        { &ei_1722_61883_incorrect_qi2,         { "iec61883.incorrect_qi2", PI_PROTOCOL, PI_WARN,
                                                   "Incorrect quadlet indicator 2 field, talker shall set this field to 0x02", EXPFILL }},
-        { &ei_1722_61883_unknown_format,        { "61883.unknown_format", PI_PROTOCOL, PI_NOTE,
+        { &ei_1722_61883_unknown_format,        { "iec61883.unknown_format", PI_PROTOCOL, PI_NOTE,
                                                   "IEC 61883 format not dissected yet", EXPFILL }},
-        { &ei_1722_61883_incorrect_channel_sid, { "61883.incorrect_channel_sid", PI_PROTOCOL, PI_WARN,
+        { &ei_1722_61883_incorrect_channel_sid, { "iec61883.incorrect_channel_sid", PI_PROTOCOL, PI_WARN,
                                                   "1394 Packet Channel and Source ID don`t match", EXPFILL }},
-        { &ei_1722_61883_incorrect_datalen,     { "61883.incorrect_datalen", PI_PROTOCOL, PI_WARN,
+        { &ei_1722_61883_incorrect_datalen,     { "iec61883.incorrect_datalen", PI_PROTOCOL, PI_WARN,
                                                   "Incorrect stream data length field, must be multiple of 192 plus 8 bytes CIP header", EXPFILL }},
-        { &ei_1722_61883_4_incorrect_cip_fn,    { "61883.4_incorrect_cip_fn", PI_PROTOCOL, PI_WARN,
+        { &ei_1722_61883_4_incorrect_cip_fn,    { "iec61883.4_incorrect_cip_fn", PI_PROTOCOL, PI_WARN,
                                                   "Incorrect fraction number, shall be 8 for IEC 61883-4", EXPFILL }},
-        { &ei_1722_61883_4_incorrect_cip_dbs,   { "61883.4_incorrect_cip_dbs", PI_PROTOCOL, PI_WARN,
+        { &ei_1722_61883_4_incorrect_cip_dbs,   { "iec61883.4_incorrect_cip_dbs", PI_PROTOCOL, PI_WARN,
                                                   "Incorrect data block size, shall be 6 for IEC 61883-4", EXPFILL }},
-        { &ei_1722_61883_4_incorrect_cip_sph,   { "61883.4_incorrect_cip_sph", PI_PROTOCOL, PI_WARN,
+        { &ei_1722_61883_4_incorrect_cip_sph,   { "iec61883.4_incorrect_cip_sph", PI_PROTOCOL, PI_WARN,
                                                   "Incorrect source packet header value, shall be 1 for IEC 61883-4", EXPFILL }},
-        { &ei_1722_61883_6_incorrect_cip_fn,    { "61883.6_incorrect_cip_fn", PI_PROTOCOL, PI_WARN,
+        { &ei_1722_61883_6_incorrect_cip_fn,    { "iec61883.6_incorrect_cip_fn", PI_PROTOCOL, PI_WARN,
                                                   "Incorrect fraction number, shall be 0 for IEC 61883-6", EXPFILL }},
-        { &ei_1722_61883_6_incorrect_cip_sph,   { "61883.6_incorrect_cip_sph", PI_PROTOCOL, PI_WARN,
+        { &ei_1722_61883_6_incorrect_cip_sph,   { "iec61883.6_incorrect_cip_sph", PI_PROTOCOL, PI_WARN,
                                                   "Incorrect source packet header value, shall be 0 for IEC 61883-6", EXPFILL }},
-        { &ei_1722_61883_incorrect_cip_fdf,     { "61883.6_incorrect_cip_fdf", PI_PROTOCOL, PI_WARN,
+        { &ei_1722_61883_incorrect_cip_fdf,     { "iec61883.6_incorrect_cip_fdf", PI_PROTOCOL, PI_WARN,
                                                   "Incorrect frame dependent field value, shall be 0", EXPFILL }}
     };
 
@@ -972,7 +1073,7 @@ void proto_register_1722_61883(void)
     proto_1722_61883 = proto_register_protocol(
                 "IEC 61883 Protocol",   /* name */
                 "IEC 61883",            /* short name */
-                "61883");               /* abbrev */
+                "iec61883");            /* abbrev */
 
     /* Required function calls to register the header fields and subtrees used */
     proto_register_field_array(proto_1722_61883, hf, array_length(hf));
@@ -1220,6 +1321,213 @@ void proto_reg_handoff_1722_aaf(void)
     dissector_add_uint("ieee1722.subtype", IEEE_1722_SUBTYPE_AAF, avb1722_aaf_handle);
 }
 
+/**************************************************************************************************/
+/* 1722 CVF dissector implementation                                                              */
+/*                                                                                                */
+/**************************************************************************************************/
+static int dissect_1722_cvf (tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, void* data _U_)
+{
+    proto_item *ti;
+    proto_tree *ti_cvf_tree;
+    gint        offset = 1;
+    guint       format_subtype = 0;
+    proto_tree *ti_format;
+
+    const gint *fields[] = {
+        &hf_1722_cvf_mrfield,
+        &hf_1722_cvf_tvfield,
+        NULL
+    };
+    const gint *fields_cvf[] = {
+        &hf_1722_cvf_marker_bit,
+        &hf_1722_cvf_evtfield,
+        NULL
+    };
+
+    const gint *fields_h264[] = {
+        &hf_1722_cvf_h264_forbidden_bit,
+        &hf_1722_cvf_h264_nri,
+        &hf_1722_cvf_h264_nal_type,
+        NULL
+    };
+
+    ti = proto_tree_add_item(tree, proto_1722_cvf, tvb, 0, -1, ENC_NA);
+    ti_cvf_tree = proto_item_add_subtree(ti, ett_1722_cvf);
+
+    proto_tree_add_bitmask_list(ti_cvf_tree, tvb, offset, 1, fields, ENC_NA);
+    offset += 1;
+    proto_tree_add_item(ti_cvf_tree, hf_1722_cvf_seqnum, tvb, offset, 1, ENC_BIG_ENDIAN);
+    offset += 1;
+    proto_tree_add_item(ti_cvf_tree, hf_1722_cvf_tufield, tvb, offset, 1, ENC_BIG_ENDIAN);
+    offset += 1;
+    proto_tree_add_item(ti_cvf_tree, hf_1722_cvf_stream_id, tvb, offset, 8, ENC_BIG_ENDIAN);
+    offset += 8;
+    proto_tree_add_item(ti_cvf_tree, hf_1722_cvf_avtp_timestamp, tvb, offset, 4, ENC_BIG_ENDIAN);
+    offset += 4;
+    proto_tree_add_item(ti_cvf_tree, hf_1722_cvf_format, tvb, offset, 1, ENC_BIG_ENDIAN);
+    offset += 1;
+    ti_format = proto_tree_add_item_ret_uint(ti_cvf_tree, hf_1722_cvf_format_subtype, tvb, offset, 1, ENC_BIG_ENDIAN, &format_subtype);
+    offset += 3;
+    proto_tree_add_item(ti_cvf_tree, hf_1722_cvf_stream_data_length, tvb, offset, 2, ENC_BIG_ENDIAN);
+    offset += 2;
+    proto_tree_add_bitmask_list(ti_cvf_tree, tvb, offset, 1, fields_cvf, ENC_BIG_ENDIAN);
+    offset += 2;
+
+    switch(format_subtype) {
+    case IEEE_1722_CVF_FORMAT_SUBTYPE_MJPEG:
+        proto_tree_add_item(ti_cvf_tree, hf_1722_cvf_mjpeg_ts, tvb, offset, 1, ENC_BIG_ENDIAN);
+        offset += 1;
+        proto_tree_add_item(ti_cvf_tree, hf_1722_cvf_mjpeg_fragment_offset, tvb, offset, 3, ENC_BIG_ENDIAN);
+        offset += 3;
+        proto_tree_add_item(ti_cvf_tree, hf_1722_cvf_mjpeg_type, tvb, offset, 1, ENC_BIG_ENDIAN);
+        offset += 1;
+        proto_tree_add_item(ti_cvf_tree, hf_1722_cvf_mjpeg_q, tvb, offset, 1, ENC_BIG_ENDIAN);
+        offset += 1;
+        proto_tree_add_item(ti_cvf_tree, hf_1722_cvf_mjpeg_width, tvb, offset, 1, ENC_BIG_ENDIAN);
+        offset += 1;
+        proto_tree_add_item(ti_cvf_tree, hf_1722_cvf_mjpeg_height, tvb, offset, 1, ENC_BIG_ENDIAN);
+        break;
+
+    case IEEE_1722_CVF_FORMAT_SUBTYPE_H264:
+        proto_tree_add_item(ti_cvf_tree, hf_1722_cvf_h264_timestamp, tvb, offset, 4, ENC_BIG_ENDIAN);
+        offset += 4;
+        proto_tree_add_bitmask_list(ti_cvf_tree, tvb, offset, 1, fields_h264, ENC_BIG_ENDIAN);
+        break;
+
+    case IEEE_1722_CVF_FORMAT_SUBTYPE_JPEG2000:
+        expert_add_info(pinfo, ti_format, &ei_cvf_jpeg2000_format);
+        break;
+
+    default:
+        expert_add_info(pinfo, ti_format, &ei_cvf_reserved_format);
+        break;
+    }
+    return tvb_captured_length(tvb);
+}
+
+void proto_register_1722_cvf (void)
+{
+    static hf_register_info hf[] =
+    {
+        { &hf_1722_cvf_mrfield,
+            { "Media Clock Restart", "cvf.mrfield",
+              FT_BOOLEAN, 8, NULL, IEEE_1722_MR_MASK, NULL, HFILL }
+        },
+        { &hf_1722_cvf_tvfield,
+            { "Source Timestamp Valid", "cvf.tvfield",
+              FT_BOOLEAN, 8, NULL, IEEE_1722_TV_MASK, NULL, HFILL }
+            },
+        { &hf_1722_cvf_seqnum,
+            { "Sequence Number", "cvf.seqnum",
+              FT_UINT8, BASE_DEC, NULL, IEEE_1722_SEQ_NUM_MASK, NULL, HFILL }
+        },
+        { &hf_1722_cvf_tufield,
+            { "Timestamp Uncertain", "cvf.tufield",
+              FT_BOOLEAN, 8, NULL, IEEE_1722_TU_MASK, NULL, HFILL }
+        },
+        { &hf_1722_cvf_stream_id,
+            { "Stream ID", "cvf.stream_id",
+              FT_UINT64, BASE_HEX, NULL, IEEE_1722_STREAM_ID_MASK, NULL, HFILL }
+        },
+        { &hf_1722_cvf_avtp_timestamp,
+            { "AVTP Timestamp", "cvf.avtp_timestamp",
+              FT_UINT32, BASE_DEC, NULL, IEEE_1722_TIMESTAMP_MASK, NULL, HFILL }
+        },
+        { &hf_1722_cvf_format,
+            { "Format", "cvf.format",
+              FT_UINT8, BASE_HEX | BASE_RANGE_STRING, RVALS(cvf_format_range_rvals), IEEE_1722_FORMAT_MASK, NULL, HFILL }
+        },
+        { &hf_1722_cvf_format_subtype,
+            { "CVF Format Subtype", "cvf.format_subtype",
+              FT_UINT8, BASE_HEX | BASE_RANGE_STRING, RVALS(cvf_format_subtype_range_rvals), IEEE_1722_FORMAT_SUBTYPE_MASK, NULL, HFILL }
+        },
+        { &hf_1722_cvf_stream_data_length,
+            { "Stream Data Length", "cvf.stream_data_len",
+              FT_UINT16, BASE_DEC | BASE_UNIT_STRING, &units_byte_bytes, IEEE_1722_STREAM_DATA_LENGTH_MASK, NULL, HFILL }
+        },
+        { &hf_1722_cvf_marker_bit,
+            { "Marker Bit", "cvf.marker_bit",
+              FT_BOOLEAN, 8, TFS(&tfs_marker_bit_set_not_set), IEEE_1722_MARKER_BIT_MASK, NULL, HFILL }
+        },
+        { &hf_1722_cvf_evtfield,
+            { "EVT", "cvf.evtfield",
+              FT_UINT8, BASE_HEX, NULL, IEEE_1722_EVT_MASK, NULL, HFILL }
+        },
+        { &hf_1722_cvf_h264_timestamp,
+            { "H264 Timestamp", "cvf.h264_timestamp",
+              FT_UINT32, BASE_DEC, NULL, IEEE_1722_CVF_H264_TIMESTAMP_MASK, NULL, HFILL }
+        },
+        { &hf_1722_cvf_h264_forbidden_bit,
+            { "H264 Forbidden Bit", "cvf.h264_f_bit",
+              FT_BOOLEAN, 8, NULL, IEEE_1722_CVF_H264_FORBIDDEN_MASK, NULL, HFILL }
+        },
+        { &hf_1722_cvf_h264_nri,
+            { "H264 NRI", "cvf.h264_nri",
+              FT_UINT8, BASE_HEX, NULL, IEEE_1722_CVF_H264_NRI_MASK, NULL, HFILL }
+        },
+        { &hf_1722_cvf_h264_nal_type,
+            { "H264 NAL Type", "cvf.h264_nal_type",
+              FT_UINT8, BASE_HEX | BASE_RANGE_STRING, RVALS(cvf_h264_nal_type_range_rvals), IEEE_1722_CVF_H264_NAL_TYPE_MASK, NULL, HFILL }
+        },
+        { &hf_1722_cvf_mjpeg_ts,
+            { "MJPEG Type Specific", "cvf.mjpeg_type_specific",
+              FT_UINT8, BASE_DEC | BASE_RANGE_STRING, RVALS(cvf_mjpeg_ts_range_rvals), IEEE_1722_CVF_MJPEG_TS_MASK, NULL, HFILL }
+        },
+        { &hf_1722_cvf_mjpeg_fragment_offset,
+            { "MJPEG Fragment Offset", "cvf.mjpeg_fragment_offset",
+              FT_UINT24, BASE_DEC, NULL, IEEE_1722_CVF_MJPEG_FRAGMENT_OFFSET_MASK, NULL, HFILL }
+        },
+        { &hf_1722_cvf_mjpeg_type,
+            { "MJPEG Type", "cvf.mjpeg_type",
+              FT_UINT8, BASE_DEC, NULL, IEEE_1722_CVF_MJPEG_TYPE_MASK, NULL, HFILL }
+        },
+        { &hf_1722_cvf_mjpeg_q,
+            { "MJPEG Q", "cvf.mjpeg_q",
+               FT_UINT8, BASE_DEC, NULL, IEEE_1722_CVF_MJPEG_Q_MASK, NULL, HFILL }
+        },
+        { &hf_1722_cvf_mjpeg_width,
+            { "MJPEG Width", "cvf.mjpeg_width",
+               FT_UINT8, BASE_DEC, NULL, IEEE_1722_CVF_MJPEG_WIDTH_MASK, NULL, HFILL }
+        },
+        { &hf_1722_cvf_mjpeg_height,
+            { "MJPEG Height", "cvf.mjpeg_height",
+               FT_UINT8, BASE_DEC, NULL, IEEE_1722_CVF_MJPEG_HEIGHT_MASK, NULL, HFILL }
+        },
+
+    };
+
+    static ei_register_info ei[] = {
+        { &ei_cvf_jpeg2000_format,          { "cvf.expert.jpeg2000_video", PI_PROTOCOL, PI_WARN, "JPEG2000 format is currently not supported", EXPFILL }},
+        { &ei_cvf_reserved_format,          { "cvf.expert.reserved_format", PI_PROTOCOL, PI_WARN, "Incorrect format, can`t be dissected", EXPFILL }}
+    };
+
+    static gint *ett[] =
+    {
+        &ett_1722_cvf,
+    };
+
+    expert_module_t *expert_1722_cvf;
+
+    /* Register the protocol name and description */
+    proto_1722_cvf = proto_register_protocol("AVTP Compressed Video Format", "CVF", "cvf");
+
+    /* Required function calls to register the header fields and subtrees used */
+    proto_register_field_array(proto_1722_cvf, hf, array_length(hf));
+    proto_register_subtree_array(ett, array_length(ett));
+
+    expert_1722_cvf = expert_register_protocol(proto_1722_cvf);
+    expert_register_field_array(expert_1722_cvf, ei, array_length(ei));
+
+}
+
+void proto_reg_handoff_1722_cvf(void)
+{
+    dissector_handle_t avb1722_cvf_handle;
+
+    avb1722_cvf_handle = create_dissector_handle(dissect_1722_cvf, proto_1722_cvf);
+    dissector_add_uint("ieee1722.subtype", IEEE_1722_SUBTYPE_CVF, avb1722_cvf_handle);
+}
+
 /**************************************************************************************************/
 /* 1722 CRF dissector implementation                                                              */
 /*                                                                                                */
