@@ -6,19 +6,7 @@
  * By Gerald Combs <gerald@wireshark.org>
  * Copyright 1998 Gerald Combs
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ * SPDX-License-Identifier: GPL-2.0-or-later
  */
 
 /* This dissector supports the command and response apdu structure
@@ -40,6 +28,7 @@ void proto_reg_handoff_iso7816(void);
 static int proto_iso7816 = -1;
 static int proto_iso7816_atr = -1;
 
+static dissector_handle_t iso7816_handle;
 static dissector_handle_t iso7816_atr_handle;
 
 static wmem_tree_t *transactions = NULL;
@@ -927,7 +916,7 @@ proto_register_iso7816(void)
     expert_iso7816 = expert_register_protocol(proto_iso7816);
     expert_register_field_array(expert_iso7816, ei, array_length(ei));
 
-    register_dissector("iso7816", dissect_iso7816, proto_iso7816);
+    iso7816_handle = register_dissector("iso7816", dissect_iso7816, proto_iso7816);
 
     transactions = wmem_tree_new_autoreset(wmem_epan_scope(), wmem_file_scope());
 
@@ -935,6 +924,13 @@ proto_register_iso7816(void)
     iso7816_atr_handle = register_dissector("iso7816.atr", dissect_iso7816_atr, proto_iso7816_atr);
 }
 
+
+void proto_reg_handoff_iso7816(void)
+{
+    dissector_add_for_decode_as("usbccid.subdissector", iso7816_handle);
+}
+
+
 /*
  * Editor modelines  -  http://www.wireshark.org/tools/modelines.html
  *
