@@ -6,19 +6,7 @@
  * By Gerald Combs <gerald@wireshark.org>
  * Copyright 1998 Gerald Combs
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ * SPDX-License-Identifier: GPL-2.0-or-later
  */
 
 #include "config.h"
@@ -35,6 +23,7 @@
 #include <epan/ppptypes.h>
 #include <epan/arcnet_pids.h>
 #include <epan/nlpid.h>
+#include <epan/addr_resolv.h>
 #include "packet-fc.h"
 #include "packet-ip.h"
 #include "packet-ipx.h"
@@ -499,6 +488,7 @@ dissect_snap(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree,
 	     int bridge_pad)
 {
 	guint32		oui;
+	const gchar *oui_str;
 	guint16		etype;
 	tvbuff_t	*next_tvb;
 	oui_info_t	*oui_info;
@@ -509,14 +499,14 @@ dissect_snap(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree,
 	/*
 	 * XXX - what about non-UI frames?
 	 */
-	oui =	tvb_get_ntoh24(tvb, offset);
 	etype = tvb_get_ntohs(tvb, offset+3);
 
+	proto_tree_add_item_ret_uint(snap_tree, hf_oui, tvb, offset, 3, ENC_BIG_ENDIAN, &oui);
+	oui_str = uint_get_manuf_name_if_known(oui);
+
 	col_append_fstr(pinfo->cinfo, COL_INFO,
 		    "; SNAP, OUI 0x%06X (%s), PID 0x%04X",
-		    oui, val_to_str_const(oui, oui_vals, "Unknown"), etype);
-
-	proto_tree_add_uint(snap_tree, hf_oui, tvb, offset, 3, oui);
+		    oui, oui_str ? oui_str : "Unknown", etype);
 
 	switch (oui) {
 
@@ -792,8 +782,8 @@ proto_register_llc(void)
 			VALS(etype_vals), 0x0, NULL, HFILL }},
 
 		{ &hf_llc_oui,
-		{ "Organization Code",	"llc.oui", FT_UINT24, BASE_HEX,
-			VALS(oui_vals), 0x0, NULL, HFILL }},
+		{ "Organization Code",	"llc.oui", FT_UINT24, BASE_OUI,
+			NULL, 0x0, NULL, HFILL }},
 
 		{ &hf_llc_pid,
 		{ "Protocol ID", "llc.pid", FT_UINT16, BASE_HEX,
