@@ -33,19 +33,7 @@
  * By Gerald Combs <gerald@wireshark.org>
  * Copyright 1998 Gerald Combs
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ * SPDX-License-Identifier: GPL-2.0-or-later
  */
 
 #include "config.h"
@@ -189,6 +177,7 @@ static const value_string mpls_pwac_types[] = {
     { 0x0025, "On-Demand CV"},
     { 0x0026, "LI"},
     { 0x0027, "Pseudo-Wire OAM"},
+    { 0x0028, "MAC Withdraw OAM Msg"},
     { 0x0057, "IPv6 packet" },
     { 0x0058, "Fault OAM"},
     { 0x7FF8, "Reserved for Experimental Use"},
@@ -282,11 +271,8 @@ dissect_pw_ach(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, void* data _
         if (res != 0)
             expert_add_info(pinfo, ti, &ei_mpls_pw_ach_res);
 
-        proto_tree_add_uint_format_value(mpls_pw_ach_tree, hf_mpls_pw_ach_channel_type,
-                                         tvb, 2, 2, channel_type,
-                                         "%s (0x%04x)",
-                                         val_to_str_ext_const(channel_type, &mpls_pwac_types_ext, "Unknown"),
-                                         channel_type);
+        proto_tree_add_item(mpls_pw_ach_tree, hf_mpls_pw_ach_channel_type,
+                            tvb, 2, 2, ENC_BIG_ENDIAN);
 
     } /* if (tree) */
 
@@ -580,7 +566,7 @@ proto_register_mpls(void)
 
         {&hf_mpls_pw_ach_channel_type,
          {"Channel Type", "pwach.channel_type",
-          FT_UINT16, BASE_HEX, NULL, 0x0,
+          FT_UINT16, BASE_HEX|BASE_EXT_STRING, &mpls_pwac_types_ext, 0x0,
           "PW Associated Channel Type", HFILL }
         },
 
