@@ -10,19 +10,7 @@
  *
  * Copied from packet-mysql.c
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ * SPDX-License-Identifier: GPL-2.0-or-later
  *
  *
  * Note:  support SSH v1 and v2  now.
@@ -213,7 +201,8 @@ static gboolean ssh_desegment = TRUE;
 
 static dissector_handle_t ssh_handle;
 
-#define TCP_PORT_SSH  22
+// 29418/tcp: Gerrit Code Review
+#define TCP_RANGE_SSH  "22,29418"
 #define SCTP_PORT_SSH 22
 
 /* Message Numbers (from RFC 4250) (1-255) */
@@ -1258,7 +1247,6 @@ ssh_dissect_proposal(tvbuff_t *tvb, int offset, proto_tree *tree,
     return offset;
 }
 
-
 void
 proto_register_ssh(void)
 {
@@ -1614,7 +1602,7 @@ proto_register_ssh(void)
 void
 proto_reg_handoff_ssh(void)
 {
-    dissector_add_uint_with_preference("tcp.port", TCP_PORT_SSH, ssh_handle);
+    dissector_add_uint_range_with_preference("tcp.port", TCP_RANGE_SSH, ssh_handle);
     dissector_add_uint("sctp.port", SCTP_PORT_SSH, ssh_handle);
     dissector_add_uint("sctp.ppi", SSH_PAYLOAD_PROTOCOL_ID, ssh_handle);
 }
