@@ -7,19 +7,7 @@
  * By Gerald Combs <gerald@wireshark.org>
  * Copyright 1998 Gerald Combs
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ * SPDX-License-Identifier: GPL-2.0-or-later
  *
  * Protocol ref:
  * http://tools.ietf.org/html/draft-davie-stt-07
@@ -524,12 +512,10 @@ dissect_stt(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 
 static gboolean
 dissect_stt_heur(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree,
-                 void *data)
+                 void *iph)
 {
-    ws_ip *iph = (ws_ip*)data;
-
     /* Make sure we at least have a TCP header */
-    if (iph == NULL || iph->ip_nxt != IP_PROTO_TCP ||
+    if (ws_ip_protocol(iph) != IP_PROTO_TCP ||
         tvb_captured_length(tvb) < STT_TCP_HDR_LEN) {
         return FALSE;
     }
