@@ -6,19 +6,7 @@
  * By Gerald Combs <gerald@wireshark.org>
  * Copyright 1998 Gerald Combs
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ * SPDX-License-Identifier: GPL-2.0-or-later
  *
  * References:
  * http://www.umatechnology.org/
@@ -90,7 +78,7 @@ static int hf_uma_urlc_seq_nr				= -1;
 static int hf_uma_urr_IE				= -1;
 static int hf_uma_urr_IE_len				= -1;
 static int hf_uma_urr_uri				= -1;
-static int hf_uma_urr_radio_type_of_id	= -1;
+static int hf_uma_urr_radio_type_of_id			= -1;
 static int hf_uma_urr_radio_id				= -1;
 static int hf_uma_urr_cell_id				= -1;
 static int hf_uma_urr_lac				= -1;
@@ -104,7 +92,7 @@ static int hf_uma_urr_psho				= -1;
 static int hf_uma_urr_IP_Address_type			= -1;
 static int hf_uma_urr_FQDN				= -1;
 static int hf_uma_urr_sgw_ipv4				= -1;
-static int hf_uma_urr_redirection_counter =		 -1;
+static int hf_uma_urr_redirection_counter		= -1;
 static int hf_uma_urr_dis_rej_cau			= -1;
 static int hf_uma_urr_MSCR				= -1;
 static int hf_uma_urr_ATT				= -1;
@@ -198,7 +186,6 @@ static expert_field ei_uma_unknown_format = EI_INIT;
 #define DEFAULT_UMA_PORT_RANGE "14001" /* Not IANA registered */
 
 /* Global variables */
-static	guint32		sgw_ipv4_address;
 static	guint32		unc_ipv4_address;
 /** static	guint32		rtp_ipv4_address; **/
 static	guint32		rtcp_ipv4_address;
@@ -970,8 +957,7 @@ dissect_uma_IE(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, int offset)
 		if ( octet == 0x57 ){ /* IPv6 */
 
 		}else{ /* All other values shall be interpreted as Ipv4 address in this version of the protocol.*/
-			sgw_ipv4_address = tvb_get_ipv4(tvb, ie_offset);
-			proto_tree_add_ipv4(urr_ie_tree, hf_uma_urr_sgw_ipv4, tvb, ie_offset, 4, sgw_ipv4_address);
+			proto_tree_add_item(urr_ie_tree, hf_uma_urr_sgw_ipv4, tvb, ie_offset, 4, ENC_BIG_ENDIAN);
 
 		}
 		break;
@@ -1477,14 +1463,14 @@ dissect_uma_IE(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, int offset)
 
 		set_address(&dst_addr, AT_IPv4, 4, &GPRS_user_data_ipv4_address);
 
-		conversation = find_conversation(pinfo->num,&dst_addr,
-			&null_addr, PT_UDP, GPRS_user_data_transport_UDP_port,
+		conversation = find_conversation(pinfo->num, &dst_addr,
+			&null_addr, ENDPOINT_UDP, GPRS_user_data_transport_UDP_port,
 			0, NO_ADDR_B|NO_PORT_B);
 
 		if (conversation == NULL) {
 			/* It's not part of any conversation - create a new one. */
 			conversation = conversation_new(pinfo->num, &dst_addr,
-			    &null_addr, PT_UDP,GPRS_user_data_transport_UDP_port ,
+			    &null_addr, ENDPOINT_UDP, GPRS_user_data_transport_UDP_port ,
 			    0, NO_ADDR2|NO_PORT2);
 
 		/* Set dissector */
@@ -1509,14 +1495,14 @@ dissect_uma_IE(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, int offset)
 
 		set_address(&dst_addr, AT_IPv4, 4, &unc_ipv4_address);
 
-		conversation = find_conversation(pinfo->num,&dst_addr,
-			&null_addr, PT_TCP, UNC_tcp_port,
+		conversation = find_conversation(pinfo->num, &dst_addr,
+			&null_addr, ENDPOINT_TCP, UNC_tcp_port,
 			0, NO_ADDR_B|NO_PORT_B);
 
 		if (conversation == NULL) {
 			/* It's not part of any conversation - create a new one. */
 			conversation = conversation_new(pinfo->num, &dst_addr,
-			    &null_addr, PT_TCP,UNC_tcp_port ,
+			    &null_addr, ENDPOINT_TCP, UNC_tcp_port,
 			    0, NO_ADDR2|NO_PORT2);
 			/* Set dissector */
 			conversation_set_dissector(conversation, uma_tcp_handle);
