@@ -8,19 +8,7 @@
  * By Gerald Combs <gerald@wireshark.org>
  * Copyright 1998 Gerald Combs
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ * SPDX-License-Identifier: GPL-2.0-or-later
  */
 
 /*  Include Files */
@@ -344,9 +332,11 @@ proto_register_zbee_zcl_appl_idt(void)
 void
 proto_reg_handoff_zbee_zcl_appl_idt(void)
 {
-    zbee_zcl_init_cluster(  proto_zbee_zcl_appl_idt,
+    zbee_zcl_init_cluster(  ZBEE_PROTOABBREV_ZCL_APPLIDT,
+                            proto_zbee_zcl_appl_idt,
                             ett_zbee_zcl_appl_idt,
                             ZBEE_ZCL_CID_APPLIANCE_IDENTIFICATION,
+                            ZBEE_MFG_CODE_NONE,
                             hf_zbee_zcl_appl_idt_attr_id,
                             -1, -1,
                             (zbee_zcl_fn_attr_data)dissect_zcl_appl_idt_attr_data
@@ -532,9 +522,15 @@ proto_register_zbee_zcl_met_idt(void)
 
     };
 
+    /* ZCL Meter Identification subtrees */
+    gint *ett[] = {
+        &ett_zbee_zcl_met_idt
+    };
+
     /* Register the ZigBee ZCL Meter Identification cluster protocol name and description */
     proto_zbee_zcl_met_idt = proto_register_protocol("ZigBee ZCL Meter Identification", "ZCL Meter Identification", ZBEE_PROTOABBREV_ZCL_METIDT);
     proto_register_field_array(proto_zbee_zcl_met_idt, hf, array_length(hf));
+    proto_register_subtree_array(ett, array_length(ett));
 
     /* Register the ZigBee ZCL Meter Identification dissector. */
     register_dissector(ZBEE_PROTOABBREV_ZCL_METIDT, dissect_zbee_zcl_met_idt, proto_zbee_zcl_met_idt);
@@ -547,9 +543,11 @@ proto_register_zbee_zcl_met_idt(void)
 void
 proto_reg_handoff_zbee_zcl_met_idt(void)
 {
-    zbee_zcl_init_cluster(  proto_zbee_zcl_met_idt,
+    zbee_zcl_init_cluster(  ZBEE_PROTOABBREV_ZCL_METIDT,
+                            proto_zbee_zcl_met_idt,
                             ett_zbee_zcl_met_idt,
                             ZBEE_ZCL_CID_METER_IDENTIFICATION,
+                            ZBEE_MFG_CODE_NONE,
                             hf_zbee_zcl_met_idt_attr_id,
                             -1, -1,
                             (zbee_zcl_fn_attr_data)dissect_zcl_met_idt_attr_data
@@ -920,15 +918,11 @@ proto_register_zbee_zcl_appl_evtalt(void)
 void
 proto_reg_handoff_zbee_zcl_appl_evtalt(void)
 {
-    dissector_handle_t appl_evtalt_handle;
-
-    /* Register our dissector with the ZigBee application dissectors. */
-    appl_evtalt_handle = find_dissector(ZBEE_PROTOABBREV_ZCL_APPLEVTALT);
-    dissector_add_uint("zbee.zcl.cluster", ZBEE_ZCL_CID_APPLIANCE_EVENTS_AND_ALERT, appl_evtalt_handle);
-
-    zbee_zcl_init_cluster(  proto_zbee_zcl_appl_evtalt,
+    zbee_zcl_init_cluster(  ZBEE_PROTOABBREV_ZCL_APPLEVTALT,
+                            proto_zbee_zcl_appl_evtalt,
                             ett_zbee_zcl_appl_evtalt,
                             ZBEE_ZCL_CID_APPLIANCE_EVENTS_AND_ALERT,
+                            ZBEE_MFG_CODE_NONE,
                             -1,
                             hf_zbee_zcl_appl_evtalt_srv_rx_cmd_id,
                             hf_zbee_zcl_appl_evtalt_srv_tx_cmd_id,
@@ -1278,15 +1272,11 @@ proto_register_zbee_zcl_appl_stats(void)
 void
 proto_reg_handoff_zbee_zcl_appl_stats(void)
 {
-    dissector_handle_t appl_stats_handle;
-
-    /* Register our dissector with the ZigBee application dissectors. */
-    appl_stats_handle = find_dissector(ZBEE_PROTOABBREV_ZCL_APPLSTATS);
-    dissector_add_uint("zbee.zcl.cluster", ZBEE_ZCL_CID_APPLIANCE_STATISTICS, appl_stats_handle);
-
-    zbee_zcl_init_cluster(  proto_zbee_zcl_appl_stats,
+    zbee_zcl_init_cluster(  ZBEE_PROTOABBREV_ZCL_APPLSTATS,
+                            proto_zbee_zcl_appl_stats,
                             ett_zbee_zcl_appl_stats,
                             ZBEE_ZCL_CID_APPLIANCE_STATISTICS,
+                            ZBEE_MFG_CODE_NONE,
                             hf_zbee_zcl_appl_stats_attr_id,
                             hf_zbee_zcl_appl_stats_srv_rx_cmd_id,
                             hf_zbee_zcl_appl_stats_srv_tx_cmd_id,
