@@ -662,7 +662,7 @@ extcap_pref_for_argument(const gchar *ifname, struct _extcap_arg *arg)
     struct preference *pref = NULL;
 
     GRegex *regex_name = g_regex_new("[-]+", (GRegexCompileFlags) 0, (GRegexMatchFlags) 0, NULL);
-    GRegex *regex_ifname = g_regex_new("(?![a-zA-Z1-9_]).", (GRegexCompileFlags) 0, (GRegexMatchFlags) 0, NULL);
+    GRegex *regex_ifname = g_regex_new("(?![a-zA-Z0-9_]).", (GRegexCompileFlags) 0, (GRegexMatchFlags) 0, NULL);
     if (regex_name && regex_ifname)
     {
         if (prefs_find_module("extcap"))
@@ -707,7 +707,7 @@ static gboolean cb_preference(extcap_callback_info_t cb_info)
         GList *walker = arguments;
 
         GRegex *regex_name = g_regex_new("[-]+", (GRegexCompileFlags) 0, (GRegexMatchFlags) 0, NULL);
-        GRegex *regex_ifname = g_regex_new("(?![a-zA-Z1-9_]).", (GRegexCompileFlags) 0, (GRegexMatchFlags) 0, NULL);
+        GRegex *regex_ifname = g_regex_new("(?![a-zA-Z0-9_]).", (GRegexCompileFlags) 0, (GRegexMatchFlags) 0, NULL);
         if (regex_name && regex_ifname)
         {
             while (walker != NULL)
@@ -1032,7 +1032,7 @@ void extcap_if_cleanup(capture_options *capture_opts, gchar **errormsg)
         pipedata = (ws_pipe_t *) interface_opts->extcap_pipedata;
         if (pipedata)
         {
-            if (pipedata->stderr_fd > 0 && ws_pipe_data_available(pipedata->stderr_fd))
+            if (pipedata->stderr_fd > 0)
             {
                 buffer = (gchar *)g_malloc0(STDERR_BUFFER_SIZE + 1);
                 ws_read_string_from_pipe(ws_get_pipe_handle(pipedata->stderr_fd), buffer, STDERR_BUFFER_SIZE + 1);
@@ -1088,12 +1088,19 @@ void extcap_if_cleanup(capture_options *capture_opts, gchar **errormsg)
             interface_opts->extcap_child_watch = 0;
         }
 
+        if (pipedata->stdout_fd > 0)
+        {
+            ws_close(pipedata->stdout_fd);
+        }
+
+        if (pipedata->stderr_fd > 0)
+        {
+            ws_close(pipedata->stderr_fd);
+        }
+
         if (interface_opts->extcap_pid != WS_INVALID_PID)
         {
-#ifdef _WIN32
-            TerminateProcess(interface_opts->extcap_pid, 0);
-#endif
-            g_spawn_close_pid(interface_opts->extcap_pid);
+            ws_pipe_close((ws_pipe_t *) interface_opts->extcap_pipedata);
             interface_opts->extcap_pid = WS_INVALID_PID;
 
             g_free(interface_opts->extcap_pipedata);
