@@ -7,19 +7,7 @@
  * By Gerald Combs <gerald@wireshark.org>
  * Copyright 1998 Gerald Combs
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ * SPDX-License-Identifier: GPL-2.0-or-later
  */
 
 #include "config.h"
@@ -29,6 +17,7 @@
 #include "randpkt_core/randpkt_core.h"
 #include <wsutil/strtoi.h>
 #include <wsutil/filesystem.h>
+#include <wsutil/privileges.h>
 
 #define RANDPKT_EXTCAP_INTERFACE "randpkt"
 #define RANDPKTDUMP_VERSION_MAJOR "0"
@@ -121,11 +110,14 @@ static int list_config(char *interface)
 	g_strfreev(longname_list);
 	inc++;
 
+	extcap_config_debug(&inc);
+
 	return EXIT_SUCCESS;
 }
 
 int main(int argc, char *argv[])
 {
+	char* init_progfile_dir_error;
 	int option_idx = 0;
 	int result;
 	guint16 maxbytes = 5000;
@@ -136,7 +128,6 @@ int main(int argc, char *argv[])
 	int produce_type = -1;
 	randpkt_example	*example;
 	wtap_dumper* savedump;
-	int i;
 	int ret = EXIT_FAILURE;
 
 #ifdef _WIN32
@@ -147,6 +138,22 @@ int main(int argc, char *argv[])
 	char* help_url;
 	char* help_header = NULL;
 
+	/*
+	 * Get credential information for later use.
+	 */
+	init_process_policies();
+
+	/*
+	 * Attempt to get the pathname of the directory containing the
+	 * executable file.
+	 */
+	init_progfile_dir_error = init_progfile_dir(argv[0], NULL);
+	if (init_progfile_dir_error != NULL) {
+		g_warning("Can't get pathname of directory containing the captype program: %s.",
+			init_progfile_dir_error);
+		g_free(init_progfile_dir_error);
+	}
+
 	help_url = data_file_url("randpktdump.html");
 	extcap_base_set_util_info(extcap_conf, argv[0], RANDPKTDUMP_VERSION_MAJOR, RANDPKTDUMP_VERSION_MINOR,
 		RANDPKTDUMP_VERSION_RELEASE, help_url);
@@ -180,9 +187,6 @@ int main(int argc, char *argv[])
 	attach_parent_console();
 #endif  /* _WIN32 */
 
-	for (i = 0; i < argc; i++)
-		g_debug("%s ", argv[i]);
-
 	while ((result = getopt_long(argc, argv, ":", longopts, &option_idx)) != -1) {
 		switch (result) {
 		case OPT_VERSION:
@@ -242,6 +246,8 @@ int main(int argc, char *argv[])
 		}
 	}
 
+	extcap_cmdline_debug(argv, argc);
+
 	if (optind != argc) {
 		g_warning("Invalid option: %s", argv[optind]);
 		goto end;
@@ -284,6 +290,8 @@ int main(int argc, char *argv[])
 			goto end;
 		}
 
+		wtap_init(FALSE);
+
 		if (!all_random) {
 			produce_type = randpkt_parse_type(type);
 
