@@ -6,20 +6,7 @@
  * By Gerald Combs <gerald@wireshark.org>
  * Copyright 1998 Gerald Combs
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- */
+ * SPDX-License-Identifier: GPL-2.0-or-later*/
 
 #include "config.h"
 
@@ -62,32 +49,89 @@ if_list_comparator_alph(const void *first_arg, const void *second_arg)
 static gboolean
 fill_from_ifaces (interface_t *device)
 {
-    interface_options interface_opts;
+    interface_options *interface_opts;
     guint i;
 
     for (i = 0; i < global_capture_opts.ifaces->len; i++) {
-        interface_opts = g_array_index(global_capture_opts.ifaces, interface_options, i);
-        if (strcmp(interface_opts.name, device->name) != 0) {
+        interface_opts = &g_array_index(global_capture_opts.ifaces, interface_options, i);
+        if (strcmp(interface_opts->name, device->name) != 0) {
             continue;
         }
 
 #if defined(HAVE_PCAP_CREATE)
-        device->buffer = interface_opts.buffer_size;
-        device->monitor_mode_enabled = interface_opts.monitor_mode;
+        device->buffer = interface_opts->buffer_size;
+        device->monitor_mode_enabled = interface_opts->monitor_mode;
 #endif
-        device->pmode = interface_opts.promisc_mode;
-        device->has_snaplen = interface_opts.has_snaplen;
-        device->snaplen = interface_opts.snaplen;
+        device->pmode = interface_opts->promisc_mode;
+        device->has_snaplen = interface_opts->has_snaplen;
+        device->snaplen = interface_opts->snaplen;
         g_free(device->cfilter);
-        device->cfilter = g_strdup(interface_opts.cfilter);
-        if (interface_opts.linktype != -1) {
-            device->active_dlt = interface_opts.linktype;
+        device->cfilter = g_strdup(interface_opts->cfilter);
+        device->timestamp_type = g_strdup(interface_opts->timestamp_type);
+        if (interface_opts->linktype != -1) {
+            device->active_dlt = interface_opts->linktype;
         }
         return TRUE;
     }
     return FALSE;
 }
 
+static gchar *
+get_iface_display_name(const gchar *description, const if_info_t *if_info)
+{
+    /* Do we have a user-supplied description? */
+    if (description && description[0]) {
+        /*
+         * Yes - show both the user-supplied description and a name for the
+         * interface.
+         */
+#ifdef _WIN32
+        /*
+         * On Windows, if we have a friendly name, just show it
+         * rather than the name, as the name is a string made out
+         * of the device GUID, and not at all friendly.
+         */
+        gchar *if_string = if_info->friendly_name ? if_info->friendly_name : if_info->name;
+        return g_strdup_printf("%s: %s", description, if_string);
+#else
+        /*
+         * On UN*X, show the interface name; it's short and somewhat
+         * friendly, and many UN*X users are used to interface names,
+         * so we should show it.
+         */
+        return g_strdup_printf("%s: %s", description, if_info->name);
+#endif
+    }
+
+    if (if_info->friendly_name) {
+        /* We have a friendly name from the OS. */
+#ifdef _WIN32
+        /*
+         * On Windows, if we have a friendly name, just show it,
+         * don't show the name, as that's a string made out of
+         * the device GUID, and not at all friendly.
+         */
+        return g_strdup_printf("%s", if_info->friendly_name);
+#else
+        /*
+         * On UN*X, if we have a friendly name, show it along
+         * with the interface name; the interface name is short
+         * and somewhat friendly, and many UN*X users are used
+         * to interface names, so we should show it.
+         */
+        return g_strdup_printf("%s: %s", if_info->friendly_name, if_info->name);
+#endif
+    }
+
+    if (if_info->vendor_description) {
+        /* We have a device description from libpcap. */
+        return g_strdup_printf("%s: %s", if_info->vendor_description, if_info->name);
+    }
+
+    /* No additional descriptions found. */
+    return g_strdup(if_info->name);
+}
+
 /*
  * Fetch the list of local interfaces with capture_interface_list()
  * and set the list of "all interfaces" in *capture_opts to include
@@ -110,7 +154,7 @@ scan_local_interfaces(void (*update_cb)(void))
     data_link_info_t  *data_link_info;
     interface_t       device;
     GString           *ip_str;
-    interface_options interface_opts;
+    interface_options *interface_opts;
     gboolean          found = FALSE;
     static gboolean   running = FALSE;
     GHashTable        *selected_devices;
@@ -175,22 +219,16 @@ scan_local_interfaces(void (*update_cb)(void))
             continue;
         }
         device.name = g_strdup(if_info->name);
-        if (if_info->friendly_name != NULL) {
-            device.friendly_name = g_strdup(if_info->friendly_name);
-        } else {
-            device.friendly_name = NULL;
-        }
+        device.friendly_name = g_strdup(if_info->friendly_name);
         device.hidden = FALSE;
-        device.locked = FALSE;
         memset(&temp, 0, sizeof(temp));
         temp.name = g_strdup(if_info->name);
         temp.friendly_name = g_strdup(if_info->friendly_name);
         temp.vendor_description = g_strdup(if_info->vendor_description);
         temp.loopback = if_info->loopback;
         temp.type = if_info->type;
-#ifdef HAVE_EXTCAP
         temp.extcap = g_strdup(if_info->extcap);
-#endif
+
         /* Is this interface hidden and, if so, should we include it anyway? */
 
         descr = capture_dev_user_descr_find(if_info->name);
@@ -304,6 +342,7 @@ scan_local_interfaces(void (*update_cb)(void))
             device.snaplen = global_capture_opts.default_options.snaplen;
         }
         device.cfilter      = g_strdup(global_capture_opts.default_options.cfilter);
+        device.timestamp_type = g_strdup(global_capture_opts.default_options.timestamp_type);
 #ifdef CAN_SET_CAPTURE_BUFFER_SIZE
         if ((device.buffer = capture_dev_user_buffersize_find(if_info->name)) == -1) {
             device.buffer = global_capture_opts.default_options.buffer_size;
@@ -318,10 +357,9 @@ scan_local_interfaces(void (*update_cb)(void))
             global_capture_opts.num_selected++;
         }
 
-#ifdef HAVE_EXTCAP
         /* Extcap devices start with no cached args */
         device.external_cap_args_settings = NULL;
-#endif
+
         if (global_capture_opts.all_ifaces->len <= count) {
             g_array_append_val(global_capture_opts.all_ifaces, device);
             count = global_capture_opts.all_ifaces->len;
@@ -342,51 +380,49 @@ scan_local_interfaces(void (*update_cb)(void))
      * the list of all interfaces (all_ifaces).
      */
     for (j = 0; j < global_capture_opts.ifaces->len; j++) {
-        interface_opts = g_array_index(global_capture_opts.ifaces, interface_options, j);
+        interface_opts = &g_array_index(global_capture_opts.ifaces, interface_options, j);
 
         found = FALSE;
         for (i = 0; i < (int)global_capture_opts.all_ifaces->len; i++) {
             device = g_array_index(global_capture_opts.all_ifaces, interface_t, i);
-            if (strcmp(device.name, interface_opts.name) == 0) {
+            if (strcmp(device.name, interface_opts->name) == 0) {
                 found = TRUE;
                 break;
             }
         }
         if (!found) {  /* new interface, maybe a pipe */
             memset(&device, 0, sizeof(device));
-            device.name         = g_strdup(interface_opts.name);
-            device.display_name = interface_opts.descr ?
-                g_strdup_printf("%s: %s", device.name, interface_opts.descr) :
+            device.name         = g_strdup(interface_opts->name);
+            device.display_name = interface_opts->descr ?
+                g_strdup_printf("%s: %s", device.name, interface_opts->descr) :
                 g_strdup(device.name);
             device.hidden       = FALSE;
             device.selected     = TRUE;
-            device.type         = interface_opts.if_type;
+            device.type         = interface_opts->if_type;
 #ifdef CAN_SET_CAPTURE_BUFFER_SIZE
-            device.buffer = interface_opts.buffer_size;
+            device.buffer = interface_opts->buffer_size;
 #endif
 #if defined(HAVE_PCAP_CREATE)
-            device.monitor_mode_enabled = interface_opts.monitor_mode;
+            device.monitor_mode_enabled = interface_opts->monitor_mode;
             device.monitor_mode_supported = FALSE;
 #endif
-            device.pmode = interface_opts.promisc_mode;
-            device.has_snaplen = interface_opts.has_snaplen;
-            device.snaplen = interface_opts.snaplen;
-            device.cfilter = g_strdup(interface_opts.cfilter);
-            device.active_dlt = interface_opts.linktype;
+            device.pmode = interface_opts->promisc_mode;
+            device.has_snaplen = interface_opts->has_snaplen;
+            device.snaplen = interface_opts->snaplen;
+            device.cfilter = g_strdup(interface_opts->cfilter);
+            device.timestamp_type = g_strdup(interface_opts->timestamp_type);
+            device.active_dlt = interface_opts->linktype;
             device.addresses    = NULL;
             device.no_addresses = 0;
             device.last_packets = 0;
             device.links        = NULL;
             device.local        = TRUE;
-            device.locked       = FALSE;
-            device.if_info.name = g_strdup(interface_opts.name);
+            device.if_info.name = g_strdup(interface_opts->name);
             device.if_info.friendly_name = NULL;
-            device.if_info.vendor_description = g_strdup(interface_opts.descr);
+            device.if_info.vendor_description = g_strdup(interface_opts->descr);
             device.if_info.addrs = NULL;
             device.if_info.loopback = FALSE;
-#ifdef HAVE_EXTCAP
-            device.if_info.extcap = g_strdup(interface_opts.extcap);
-#endif
+            device.if_info.extcap = g_strdup(interface_opts->extcap);
 
             g_array_append_val(global_capture_opts.all_ifaces, device);
             global_capture_opts.num_selected++;
@@ -432,7 +468,7 @@ hide_interface(gchar* new_hide)
 {
     gchar       *tok;
     guint       i;
-    interface_t device;
+    interface_t *device;
     gboolean    found = FALSE;
     GList       *hidden_devices = NULL, *entry;
     if (new_hide != NULL) {
@@ -441,13 +477,13 @@ hide_interface(gchar* new_hide)
         }
     }
     for (i = 0; i < global_capture_opts.all_ifaces->len; i++) {
-        device = g_array_index(global_capture_opts.all_ifaces, interface_t, i);
+        device = &g_array_index(global_capture_opts.all_ifaces, interface_t, i);
         found = FALSE;
         for (entry = hidden_devices; entry != NULL; entry = g_list_next(entry)) {
-            if (strcmp((char *)entry->data, device.name)==0) {
-                device.hidden = TRUE;
-                if (device.selected) {
-                    device.selected = FALSE;
+            if (strcmp((char *)entry->data, device->name)==0) {
+                device->hidden = TRUE;
+                if (device->selected) {
+                    device->selected = FALSE;
                     global_capture_opts.num_selected--;
                 }
                 found = TRUE;
@@ -455,10 +491,8 @@ hide_interface(gchar* new_hide)
             }
         }
         if (!found) {
-            device.hidden = FALSE;
+            device->hidden = FALSE;
         }
-        global_capture_opts.all_ifaces = g_array_remove_index(global_capture_opts.all_ifaces, i);
-        g_array_insert_val(global_capture_opts.all_ifaces, i, device);
     }
     g_list_free(hidden_devices);
     g_free(new_hide);
@@ -467,22 +501,19 @@ hide_interface(gchar* new_hide)
 void
 update_local_interfaces(void)
 {
-    interface_t device;
+    interface_t *device;
     gchar *descr;
     guint i;
 
     for (i = 0; i < global_capture_opts.all_ifaces->len; i++) {
-        device = g_array_index(global_capture_opts.all_ifaces, interface_t, i);
-        device.type = capture_dev_user_linktype_find(device.name);
-        g_free (device.display_name);
-        descr = capture_dev_user_descr_find(device.name);
-        device.display_name = get_iface_display_name(descr, &device.if_info);
+        device = &g_array_index(global_capture_opts.all_ifaces, interface_t, i);
+        device->type = capture_dev_user_linktype_find(device->name);
+        g_free(device->display_name);
+        descr = capture_dev_user_descr_find(device->name);
+        device->display_name = get_iface_display_name(descr, &device->if_info);
         g_free (descr);
-        device.hidden = prefs_is_capture_device_hidden(device.name);
-        fill_from_ifaces(&device);
-
-        global_capture_opts.all_ifaces = g_array_remove_index(global_capture_opts.all_ifaces, i);
-        g_array_insert_val(global_capture_opts.all_ifaces, i, device);
+        device->hidden = prefs_is_capture_device_hidden(device->name);
+        fill_from_ifaces(device);
     }
 }
 #endif /* HAVE_LIBPCAP */
