@@ -6,20 +6,7 @@
  * By Gerald Combs <gerald@wireshark.org>
  * Copyright 1998 Gerald Combs
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- */
+ * SPDX-License-Identifier: GPL-2.0-or-later*/
 
 #include "config.h"
 
@@ -36,6 +23,7 @@
 #include "ui/recent.h"
 
 #include <wsutil/file_util.h>
+#include <wsutil/glib-compat.h>
 
 static GList *current_profiles = NULL;
 static GList *edited_profiles = NULL;
@@ -110,13 +98,13 @@ get_profile_parent (const gchar *profilename)
     return profilename;
 }
 
-const gchar *apply_profile_changes(void)
+gchar *apply_profile_changes(void)
 {
     char        *pf_dir_path, *pf_dir_path2, *pf_filename;
     GList       *fl1, *fl2;
     profile_def *profile1, *profile2;
     gboolean     found;
-    const gchar *err_msg;
+    gchar       *err_msg;
 
     /* First validate all profile names */
     fl1 = edited_profile_list();
@@ -125,7 +113,7 @@ const gchar *apply_profile_changes(void)
         g_strstrip(profile1->name);
         if ((err_msg = profile_name_is_valid(profile1->name)) != NULL) {
             gchar *message = g_strdup_printf("%s\nProfiles unchanged.", err_msg);
-            g_free((gchar *)err_msg);
+            g_free(err_msg);
             return message;
         }
         fl1 = g_list_next(fl1);
