@@ -4,27 +4,14 @@
  * By Gerald Combs <gerald@wireshark.org>
  * Copyright 1998 Gerald Combs
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- */
+ * SPDX-License-Identifier: GPL-2.0-or-later*/
 
 #include "bluetooth_hci_summary_dialog.h"
 #include <ui_bluetooth_hci_summary_dialog.h>
 
 #include "bluetooth_device_dialog.h"
 
-#include "color_utils.h"
+#include <ui/qt/utils/color_utils.h>
 
 #include "epan/epan.h"
 #include "epan/addr_resolv.h"
@@ -35,15 +22,15 @@
 #include "epan/dissectors/packet-bthci_cmd.h"
 #include "epan/dissectors/packet-bthci_evt.h"
 
-#include <ui/qt/variant_pointer.h>
+#include <ui/qt/utils/variant_pointer.h>
 
 #include "ui/simple_dialog.h"
+#include "ui/qt/widgets/wireshark_file_dialog.h"
 
 #include <QClipboard>
 #include <QContextMenuEvent>
 #include <QPushButton>
 #include <QTreeWidget>
-#include <QFileDialog>
 
 static const int column_number_name = 0;
 static const int column_number_ogf = 1;
@@ -359,14 +346,17 @@ gboolean BluetoothHciSummaryDialog::tapPacket(void *tapinfo_ptr, packet_info *pi
     if (dialog->file_closed_)
         return FALSE;
 
+    if (pinfo->rec->rec_type != REC_TYPE_PACKET)
+        return FALSE;
+
     name = tr("Unknown");
 
-    if (pinfo->phdr->presence_flags & WTAP_HAS_INTERFACE_ID) {
+    if (pinfo->rec->presence_flags & WTAP_HAS_INTERFACE_ID) {
         gchar       *interface;
         const char  *interface_name;
 
-        interface_name = epan_get_interface_name(pinfo->epan, pinfo->phdr->interface_id);
-        interface = wmem_strdup_printf(wmem_packet_scope(), "%u: %s", pinfo->phdr->interface_id, interface_name);
+        interface_name = epan_get_interface_name(pinfo->epan, pinfo->rec->rec_header.packet_header.interface_id);
+        interface = wmem_strdup_printf(wmem_packet_scope(), "%u: %s", pinfo->rec->rec_header.packet_header.interface_id, interface_name);
 
         if (dialog->ui->interfaceComboBox->findText(interface) == -1)
             dialog->ui->interfaceComboBox->addItem(interface);
@@ -835,7 +825,7 @@ void BluetoothHciSummaryDialog::on_actionCopy_All_triggered()
 
     item = ui->tableTreeWidget->headerItem();
 
-    copy += QString("%1  %2  %3  %4  %5  %6  %7  %8  %9\n")
+    copy += QString("%1  %2  %3  %4  %5  %6  %7  %8  %9  %10\n")
             .arg(item->text(column_number_name), -60)
             .arg(item->text(column_number_ogf), -10)
             .arg(item->text(column_number_ocf), -10)
@@ -850,7 +840,7 @@ void BluetoothHciSummaryDialog::on_actionCopy_All_triggered()
     for (int i_item = 0; i_item < ui->tableTreeWidget->topLevelItemCount(); ++i_item) {
         item = ui->tableTreeWidget->topLevelItem(i_item);
 
-        copy += QString("%1  %2  %3  %4  %5  %6  %7  %8  %9\n")
+        copy += QString("%1  %2  %3  %4  %5  %6  %7  %8  %9  %10\n")
                 .arg(item->text(column_number_name), -60)
                 .arg(item->text(column_number_ogf), -10)
                 .arg(item->text(column_number_ocf), -10)
@@ -872,7 +862,7 @@ void BluetoothHciSummaryDialog::on_actionSave_as_image_triggered()
 {
     QPixmap image;
 
-    QString fileName = QFileDialog::getSaveFileName(this,
+    QString fileName = WiresharkFileDialog::getSaveFileName(this,
             tr("Save Table Image"),
             "bluetooth_hci_summary.png",
             tr("PNG Image (*.png)"));
