@@ -5,19 +5,7 @@
  * By Gerald Combs <gerald@wireshark.org>
  * Copyright 2004 Gerald Combs
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ * SPDX-License-Identifier: GPL-2.0-or-later
  */
 
 #include "config.h"
@@ -31,7 +19,7 @@
 #include <richedit.h>
 #include <strsafe.h>
 
-#include "file.h"
+#include "globals.h"
 
 #include "wsutil/file_util.h"
 #include "wsutil/str_util.h"
@@ -48,11 +36,17 @@
 #include "ui/simple_dialog.h"
 #include "ui/ssl_key_export.h"
 #include "ui/util.h"
-#include "ui/ui_util.h"
+#include "ui/ws_ui_util.h"
 #include "ui/all_files_wildcard.h"
 
 #include "file_dlg_win32.h"
 
+typedef enum {
+    merge_append,
+    merge_chrono,
+    merge_prepend
+} merge_action_e;
+
 #define FILE_OPEN_DEFAULT 1 /* All Files */
 
 #define FILE_MERGE_DEFAULT FILE_OPEN_DEFAULT
@@ -99,6 +93,57 @@ static void range_handle_wm_command(HWND dlg_hwnd, HWND ctrl, WPARAM w_param, pa
 static TCHAR *build_file_open_type_list(void);
 static TCHAR *build_file_save_type_list(GArray *savable_file_types);
 
+#ifdef DPI_AWARENESS_CONTEXT_PER_MONITOR_AWARE_V2
+typedef DPI_AWARENESS_CONTEXT (WINAPI *GetThreadDpiAwarenessContextProc)(void);
+typedef DPI_AWARENESS_CONTEXT (WINAPI *SetThreadDpiAwarenessContextProc)(DPI_AWARENESS_CONTEXT);
+
+static GetThreadDpiAwarenessContextProc GetThreadDpiAwarenessContextP;
+static SetThreadDpiAwarenessContextProc SetThreadDpiAwarenessContextP;
+static gboolean got_proc_addresses = FALSE;
+
+static gboolean get_proc_addresses(void) {
+    if (got_proc_addresses) return TRUE;
+
+    HMODULE u32_module = LoadLibrary(_T("User32.dll"));
+    if (!u32_module) {
+        got_proc_addresses = FALSE;
+        return FALSE;
+    }
+    gboolean got_all = TRUE;
+    GetThreadDpiAwarenessContextP = (GetThreadDpiAwarenessContextProc) GetProcAddress(u32_module, "GetThreadDpiAwarenessContext");
+    if (!GetThreadDpiAwarenessContextP) got_all = FALSE;
+    SetThreadDpiAwarenessContextP = (SetThreadDpiAwarenessContextProc) GetProcAddress(u32_module, "SetThreadDpiAwarenessContext");
+    if (!SetThreadDpiAwarenessContextP) got_all = FALSE;
+
+    got_proc_addresses = got_all;
+    return got_all;
+}
+
+// Enabling DPI_AWARENESS_CONTEXT_PER_MONITOR_AWARE_V2 causes issues
+// when dragging our open file dialog between differently-DPIed
+// displays. It might be time to break down and switch to common
+// item dialogs.
+HANDLE set_thread_per_monitor_v2_awareness(void) {
+    if (! get_proc_addresses()) return 0;
+#if 0
+    WCHAR info[100];
+    StringCchPrintf(info, 100,
+                    L"GetThrDpiAwarenessCtx: %d",
+                    GetThreadDpiAwarenessContextP());
+    MessageBox(NULL, info, _T("DPI info"), MB_OK);
+#endif
+    return (HANDLE) SetThreadDpiAwarenessContextP(DPI_AWARENESS_CONTEXT_PER_MONITOR_AWARE_V2);
+}
+
+void revert_thread_per_monitor_v2_awareness(HANDLE context) {
+    if (! get_proc_addresses()) return;
+    SetThreadDpiAwarenessContextP((DPI_AWARENESS_CONTEXT) context);
+}
+#else // DPI_AWARENESS_CONTEXT_PER_MONITOR_AWARE_V2
+HANDLE set_thread_per_monitor_v2_awareness(void) { return 0; }
+void revert_thread_per_monitor_v2_awareness(HANDLE context _U_) { }
+#endif // DPI_AWARENESS_CONTEXT_PER_MONITOR_AWARE_V2
+
 static int             g_filetype;
 static gboolean        g_compressed;
 static packet_range_t *g_range;
@@ -120,65 +165,6 @@ static HWND  g_sf_hwnd = NULL;
 static char *g_dfilter_str = NULL;
 static unsigned int g_format_type = WTAP_TYPE_AUTO;
 
-static int
-win32_get_ofnsize()
-{
-    gboolean bVerGE5 = FALSE;
-    int ofnsize;
-    /* Remarks on OPENFILENAME_SIZE_VERSION_400:
-    *
-    * MSDN states that OPENFILENAME_SIZE_VERSION_400 should be used with
-    * WINVER and _WIN32_WINNT >= 0x0500.
-    * Unfortunately all these are compiler constants, while the underlying is a
-    * problem based is a length check of the runtime version used.
-    *
-    * Instead of using OPENFILENAME_SIZE_VERSION_400, just malloc
-    * the OPENFILENAME size plus 12 bytes.
-    * These 12 bytes are the difference between the two versions of this struct.
-    *
-    * Interestingly this fixes a bug, so the places bar e.g. "My Documents"
-    * is displayed - which wasn't the case with the former implementation.
-    *
-    * XXX - It's unclear if this length+12 works on all supported platforms,
-    * NT4 is the question here. However, even if it fails, we must calculate
-    * the length based on the runtime, not the compiler version anyway ...
-    */
-    /* This assumption does not work when compiling with MSVC2008EE as
-    * the open dialog window does not appear.
-    * Instead detect Windows version at runtime and choose size accordingly */
-#if (_MSC_VER >= 1500)
-    /*
-    * On VS2103, GetVersionEx is deprecated. Microsoft recommend to
-    * use VerifyVersionInfo instead
-    */
-#if (_MSC_VER >= 1800)
-    OSVERSIONINFOEX osvi;
-    DWORDLONG dwlConditionMask = 0;
-    int op = VER_GREATER_EQUAL;
-    /* Initialize the OSVERSIONINFOEX structure. */
-    SecureZeroMemory(&osvi, sizeof(OSVERSIONINFOEX));
-    osvi.dwOSVersionInfoSize = sizeof(OSVERSIONINFOEX);
-    osvi.dwMajorVersion = 5;
-    /* Initialize the condition mask. */
-    VER_SET_CONDITION(dwlConditionMask, VER_MAJORVERSION, op);
-    /* Perform the test. */
-    bVerGE5=VerifyVersionInfo(
-        &osvi,
-        VER_MAJORVERSION,
-        dwlConditionMask);
-#else
-    OSVERSIONINFO osvi;
-    SecureZeroMemory(&osvi, sizeof(OSVERSIONINFO));
-    osvi.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
-    GetVersionEx(&osvi);
-    bVerGE5 = (osvi.dwMajorVersion >= 5);
-#endif /* _MSC_VER >= 1800 */
-    ofnsize = (bVerGE5)?sizeof(OPENFILENAME):OPENFILENAME_SIZE_VERSION_400;
-#else
-    ofnsize = sizeof(OPENFILENAME)+12;
-#endif /* _MSC_VER >= 1500 */
-    return ofnsize;
-}
 /*
  * According to http://msdn.microsoft.com/en-us/library/bb776913.aspx
  * we should use IFileOpenDialog and IFileSaveDialog on Windows Vista
@@ -189,8 +175,8 @@ gboolean
 win32_open_file (HWND h_wnd, GString *file_name, unsigned int *type, GString *display_filter) {
     OPENFILENAME *ofn;
     TCHAR file_name16[MAX_PATH] = _T("");
-    int ofnsize;
-    gboolean gofn_ok;
+    int ofnsize = sizeof(OPENFILENAME);
+    BOOL gofn_ok;
 
     if (!file_name || !display_filter)
         return FALSE;
@@ -205,7 +191,7 @@ win32_open_file (HWND h_wnd, GString *file_name, unsigned int *type, GString *di
         g_free(g_dfilter_str);
         g_dfilter_str = NULL;
     }
-    ofnsize = win32_get_ofnsize();
+
     ofn = g_malloc0(ofnsize);
 
     ofn->lStructSize = ofnsize;
@@ -232,7 +218,9 @@ win32_open_file (HWND h_wnd, GString *file_name, unsigned int *type, GString *di
     ofn->lpfnHook = open_file_hook_proc;
     ofn->lpTemplateName = _T("WIRESHARK_OPENFILENAME_TEMPLATE");
 
+    HANDLE save_da_ctx = set_thread_per_monitor_v2_awareness();
     gofn_ok = GetOpenFileName(ofn);
+    revert_thread_per_monitor_v2_awareness(save_da_ctx);
 
     if (gofn_ok) {
         g_string_printf(file_name, "%s", utf_16to8(file_name16));
@@ -309,7 +297,7 @@ win32_check_save_as_with_comments(HWND parent, capture_file *cf, int file_type)
     switch (response) {
 
     case IDNO: /* "No" means "Save in another format" in the first dialog */
-        /* OK, the only other format we support is pcap-ng.  Make that
+        /* OK, the only other format we support is pcapng.  Make that
            the one and only format in the combo box, and return to
            let the user continue with the dialog.
 
@@ -317,7 +305,7 @@ win32_check_save_as_with_comments(HWND parent, capture_file *cf, int file_type)
            the compressed checkbox; get the current value and restore
            it.
 
-           XXX - we know pcap-ng can be compressed; if we ever end up
+           XXX - we know pcapng can be compressed; if we ever end up
            supporting saving comments in a format that *can't* be
            compressed, such as NetMon format, we must check this. */
         /* XXX - need a compressed checkbox here! */
@@ -344,8 +332,8 @@ win32_save_as_file(HWND h_wnd, capture_file *cf, GString *file_name, int *file_t
     GArray *savable_file_types;
     OPENFILENAME *ofn;
     TCHAR  file_name16[MAX_PATH] = _T("");
-    int    ofnsize;
-    gboolean gsfn_ok;
+    int    ofnsize = sizeof(OPENFILENAME);
+    BOOL gsfn_ok;
 
     if (!file_name || !file_type || !compressed)
         return FALSE;
@@ -367,7 +355,6 @@ win32_save_as_file(HWND h_wnd, capture_file *cf, GString *file_name, int *file_t
         return FALSE;  /* shouldn't happen - the "Save As..." item should be disabled if we can't save the file */
     g_compressed = FALSE;
 
-    ofnsize = win32_get_ofnsize();
     ofn = g_malloc0(ofnsize);
 
     ofn->lStructSize = ofnsize;
@@ -391,7 +378,9 @@ win32_save_as_file(HWND h_wnd, capture_file *cf, GString *file_name, int *file_t
     ofn->lpfnHook = save_as_file_hook_proc;
     ofn->lpTemplateName = _T("WIRESHARK_SAVEASFILENAME_TEMPLATE");
 
+    HANDLE save_da_ctx = set_thread_per_monitor_v2_awareness();
     gsfn_ok = GetSaveFileName(ofn);
+    revert_thread_per_monitor_v2_awareness(save_da_ctx);
 
     if (gsfn_ok) {
         g_string_printf(file_name, "%s", utf_16to8(file_name16));
@@ -421,11 +410,8 @@ gboolean win32_save_as_statstree(HWND h_wnd, GString *file_name, int *file_type)
 {
     OPENFILENAME *ofn;
     TCHAR  file_name16[MAX_PATH] = _T("");
-    int    ofnsize;
-    gboolean gsfn_ok;
-#if (_MSC_VER >= 1500)
-    OSVERSIONINFO osvi;
-#endif
+    int    ofnsize = sizeof(OPENFILENAME);
+    BOOL gsfn_ok;
 
     if (!file_name || !file_type)
         return FALSE;
@@ -434,20 +420,7 @@ gboolean win32_save_as_statstree(HWND h_wnd, GString *file_name, int *file_type)
         StringCchCopy(file_name16, MAX_PATH, utf_8to16(file_name->str));
     }
 
-    /* see OPENFILENAME comment in win32_open_file */
-#if (_MSC_VER >= 1500)
-    SecureZeroMemory(&osvi, sizeof(OSVERSIONINFO));
-    osvi.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
-    GetVersionEx(&osvi);
-    if (osvi.dwMajorVersion >= 5) {
-        ofnsize = sizeof(OPENFILENAME);
-    } else {
-        ofnsize = OPENFILENAME_SIZE_VERSION_400;
-    }
-#else
-    ofnsize = sizeof(OPENFILENAME) + 12;
-#endif
-    ofn = g_malloc0(ofnsize);
+    ofn = g_malloc0(sizeof(OPENFILENAME));
 
     ofn->lStructSize = ofnsize;
     ofn->hwndOwner = h_wnd;
@@ -469,7 +442,9 @@ gboolean win32_save_as_statstree(HWND h_wnd, GString *file_name, int *file_type)
     ofn->lpfnHook = save_as_statstree_hook_proc;
     ofn->lpTemplateName = _T("WIRESHARK_SAVEASSTATSTREENAME_TEMPLATE");
 
+    HANDLE save_da_ctx = set_thread_per_monitor_v2_awareness();
     gsfn_ok = GetSaveFileName(ofn);
+    revert_thread_per_monitor_v2_awareness(save_da_ctx);
 
     if (gsfn_ok) {
         g_string_printf(file_name, "%s", utf_16to8(file_name16));
@@ -492,8 +467,8 @@ win32_export_specified_packets_file(HWND h_wnd, capture_file *cf,
     GArray *savable_file_types;
     OPENFILENAME *ofn;
     TCHAR  file_name16[MAX_PATH] = _T("");
-    int    ofnsize;
-    gboolean gsfn_ok;
+    int    ofnsize = sizeof(OPENFILENAME);
+    BOOL gsfn_ok;
 
     if (!file_name || !file_type || !compressed || !range)
         return FALSE;
@@ -511,7 +486,6 @@ win32_export_specified_packets_file(HWND h_wnd, capture_file *cf,
     g_cf = cf;
     g_compressed = FALSE;
 
-    ofnsize = win32_get_ofnsize();
     ofn = g_malloc0(ofnsize);
 
     ofn->lStructSize = ofnsize;
@@ -535,7 +509,9 @@ win32_export_specified_packets_file(HWND h_wnd, capture_file *cf,
     ofn->lpfnHook = export_specified_packets_file_hook_proc;
     ofn->lpTemplateName = _T("WIRESHARK_EXPORT_SPECIFIED_PACKETS_FILENAME_TEMPLATE");
 
+    HANDLE save_da_ctx = set_thread_per_monitor_v2_awareness();
     gsfn_ok = GetSaveFileName(ofn);
+    revert_thread_per_monitor_v2_awareness(save_da_ctx);
 
     if (gsfn_ok) {
         g_string_printf(file_name, "%s", utf_16to8(file_name16));
@@ -568,8 +544,8 @@ gboolean
 win32_merge_file (HWND h_wnd, GString *file_name, GString *display_filter, int *merge_type) {
     OPENFILENAME *ofn;
     TCHAR         file_name16[MAX_PATH] = _T("");
-    int           ofnsize;
-    gboolean gofn_ok;
+    int           ofnsize = sizeof(OPENFILENAME);
+    BOOL          gofn_ok;
 
     if (!file_name || !display_filter || !merge_type)
         return FALSE;
@@ -585,7 +561,6 @@ win32_merge_file (HWND h_wnd, GString *file_name, GString *display_filter, int *
         g_dfilter_str = NULL;
     }
 
-    ofnsize = win32_get_ofnsize();
     ofn = g_malloc0(ofnsize);
 
     ofn->lStructSize = ofnsize;
@@ -612,7 +587,9 @@ win32_merge_file (HWND h_wnd, GString *file_name, GString *display_filter, int *
     ofn->lpfnHook = merge_file_hook_proc;
     ofn->lpTemplateName = _T("WIRESHARK_MERGEFILENAME_TEMPLATE");
 
+    HANDLE save_da_ctx = set_thread_per_monitor_v2_awareness();
     gofn_ok = GetOpenFileName(ofn);
+    revert_thread_per_monitor_v2_awareness(save_da_ctx);
 
     if (gofn_ok) {
         g_string_printf(file_name, "%s", utf_16to8(file_name16));
@@ -646,11 +623,10 @@ win32_export_file(HWND h_wnd, capture_file *cf, export_type_e export_type) {
     TCHAR             file_name[MAX_PATH] = _T("");
     char             *dirname;
     cf_print_status_t status;
-    int               ofnsize;
+    int               ofnsize = sizeof(OPENFILENAME);
 
     g_cf = cf;
 
-    ofnsize = win32_get_ofnsize();
     ofn = g_malloc0(ofnsize);
 
     ofn->lStructSize = ofnsize;
@@ -686,7 +662,11 @@ win32_export_file(HWND h_wnd, capture_file *cf, export_type_e export_type) {
     print_args.print_formfeed      = FALSE;
     print_args.stream              = NULL;
 
-    if (GetSaveFileName(ofn)) {
+    HANDLE save_da_ctx = set_thread_per_monitor_v2_awareness();
+    BOOL gsfn_ok = GetSaveFileName(ofn);
+    revert_thread_per_monitor_v2_awareness(save_da_ctx);
+
+    if (gsfn_ok) {
         print_args.file = utf_16to8(file_name);
         switch (ofn->nFilterIndex) {
             case export_type_text:      /* Text */
@@ -754,7 +734,7 @@ win32_export_raw_file(HWND h_wnd, capture_file *cf) {
     const guint8 *data_p;
     char         *file_name8;
     int           fd;
-    int           ofnsize;
+    int           ofnsize = sizeof(OPENFILENAME);
 
     if (!cf->finfo_selected) {
         /* This shouldn't happen */
@@ -762,7 +742,6 @@ win32_export_raw_file(HWND h_wnd, capture_file *cf) {
         return;
     }
 
-    ofnsize = win32_get_ofnsize();
     ofn = g_malloc0(ofnsize);
 
     ofn->lStructSize = ofnsize;
@@ -791,7 +770,11 @@ win32_export_raw_file(HWND h_wnd, capture_file *cf) {
      * grab the info from cf->finfo_selected.  Which is more "correct"?
      */
 
-    if (GetSaveFileName(ofn)) {
+    HANDLE save_da_ctx = set_thread_per_monitor_v2_awareness();
+    BOOL gsfn_ok = GetSaveFileName(ofn);
+    revert_thread_per_monitor_v2_awareness(save_da_ctx);
+
+    if (gsfn_ok) {
         g_free( (void *) ofn);
         file_name8 = utf_16to8(file_name);
         data_p = tvb_get_ptr(cf->finfo_selected->ds_tvb, 0, -1) +
@@ -827,7 +810,7 @@ win32_export_sslkeys_file(HWND h_wnd) {
     gchar        *keylist = NULL;
     char         *file_name8;
     int           fd;
-    int           ofnsize;
+    int           ofnsize = sizeof(OPENFILENAME);
     int           keylist_size;
 
     keylist_size = ssl_session_key_count();
@@ -837,7 +820,6 @@ win32_export_sslkeys_file(HWND h_wnd) {
         return;
     }
 
-    ofnsize = win32_get_ofnsize();
     ofn = g_malloc0(ofnsize);
 
     ofn->lStructSize = ofnsize;
@@ -861,7 +843,11 @@ win32_export_sslkeys_file(HWND h_wnd) {
     ofn->lpfnHook = export_sslkeys_file_hook_proc;
     ofn->lpTemplateName = _T("WIRESHARK_EXPORTSSLKEYSFILENAME_TEMPLATE");
 
-    if (GetSaveFileName(ofn)) {
+    HANDLE save_da_ctx = set_thread_per_monitor_v2_awareness();
+    BOOL gsfn_ok = GetSaveFileName(ofn);
+    revert_thread_per_monitor_v2_awareness(save_da_ctx);
+
+    if (gsfn_ok) {
         g_free( (void *) ofn);
         file_name8 = utf_16to8(file_name);
         keylist = ssl_export_sessions();
@@ -901,10 +887,9 @@ win32_export_color_file(HWND h_wnd, capture_file *cf, gpointer filter_list) {
     OPENFILENAME *ofn;
     TCHAR  file_name[MAX_PATH] = _T("");
     gchar *dirname;
-    int    ofnsize;
+    int    ofnsize = sizeof(OPENFILENAME);
     gchar *err_msg = NULL;
 
-    ofnsize = win32_get_ofnsize();
     ofn = g_malloc0(ofnsize);
 
     ofn->lStructSize = ofnsize;
@@ -930,7 +915,11 @@ win32_export_color_file(HWND h_wnd, capture_file *cf, gpointer filter_list) {
     g_filetype = cf->cd_t;
 
     /* XXX - Support marked filters */
-    if (GetSaveFileName(ofn)) {
+    HANDLE save_da_ctx = set_thread_per_monitor_v2_awareness();
+    BOOL gsfn_ok = GetSaveFileName(ofn);
+    revert_thread_per_monitor_v2_awareness(save_da_ctx);
+
+    if (gsfn_ok) {
         g_free( (void *) ofn);
         if (!color_filters_export(utf_16to8(file_name), filter_list, FALSE /* all filters */, &err_msg))
         {
@@ -952,10 +941,9 @@ win32_import_color_file(HWND h_wnd, gpointer color_filters) {
     OPENFILENAME *ofn;
     TCHAR  file_name[MAX_PATH] = _T("");
     gchar *dirname;
-    int    ofnsize;
+    int    ofnsize = sizeof(OPENFILENAME);
     gchar *err_msg = NULL;
 
-    ofnsize = win32_get_ofnsize();
     ofn = g_malloc0(ofnsize);
 
     ofn->lStructSize = ofnsize;
@@ -979,7 +967,11 @@ win32_import_color_file(HWND h_wnd, gpointer color_filters) {
     ofn->lpTemplateName = NULL;
 
     /* XXX - Support export limited to selected filters */
-    if (GetOpenFileName(ofn)) {
+    HANDLE save_da_ctx = set_thread_per_monitor_v2_awareness();
+    BOOL gofn_ok = GetOpenFileName(ofn);
+    revert_thread_per_monitor_v2_awareness(save_da_ctx);
+
+    if (gofn_ok) {
         g_free( (void *) ofn);
         if (!color_filters_import(utf_16to8(file_name), color_filters, &err_msg, color_filter_add_cb)) {
             simple_dialog(ESD_TYPE_ERROR, ESD_BTN_OK, "%s", err_msg);
@@ -1117,24 +1109,17 @@ preview_set_file_info(HWND of_hwnd, gchar *preview_file) {
     HWND        cur_ctrl;
     int         i;
     wtap       *wth;
-    const struct wtap_pkthdr *phdr;
-    int         err = 0;
+    int         err;
     gchar      *err_info;
+    ws_file_preview_stats stats;
+    ws_file_preview_stats_status status;
     TCHAR       string_buff[PREVIEW_STR_MAX];
     TCHAR       first_buff[PREVIEW_STR_MAX];
-    gint64      data_offset;
-    guint       packet = 0;
     gint64      filesize;
     gchar      *size_str;
     time_t      ti_time;
     struct tm  *ti_tm;
     guint       elapsed_time;
-    time_t      time_preview;
-    time_t      time_current;
-    double      start_time = 0;
-    double      stop_time = 0;
-    double      cur_time;
-    gboolean    is_breaked = FALSE;
 
     for (i = EWFD_PTX_FORMAT; i <= EWFD_PTX_START_ELAPSED; i++) {
         cur_ctrl = GetDlgItem(of_hwnd, i);
@@ -1188,33 +1173,13 @@ preview_set_file_info(HWND of_hwnd, gchar *preview_file) {
     // Windows Explorer uses IEC.
     size_str = format_size(filesize, format_size_unit_bytes|format_size_prefix_iec);
 
-    time(&time_preview);
-    while ( (wtap_read(wth, &err, &err_info, &data_offset)) ) {
-        phdr = wtap_phdr(wth);
-        cur_time = nstime_to_sec( (const nstime_t *) &phdr->ts );
-        if(packet == 0) {
-            start_time  = cur_time;
-            stop_time = cur_time;
-        }
-        if (cur_time < start_time) {
-            start_time = cur_time;
-        }
-        if (cur_time > stop_time){
-            stop_time = cur_time;
-        }
-        packet++;
-        if(packet%100 == 0) {
-            time(&time_current);
-            if(time_current-time_preview >= (time_t) prefs.gui_fileopen_preview) {
-                is_breaked = TRUE;
-                break;
-            }
-        }
-    }
+    status = get_stats_for_preview(wth, &stats, &err, &err_info);
 
-    if(err != 0) {
-        utf_8to16_snprintf(string_buff, PREVIEW_STR_MAX, "%s, error after %u packets",
-            size_str, packet);
+    if(status == PREVIEW_READ_ERROR) {
+        /* XXX - give error details? */
+        g_free(err_info);
+        utf_8to16_snprintf(string_buff, PREVIEW_STR_MAX, "%s, error after %u records",
+            size_str, stats.records);
         g_free(size_str);
         cur_ctrl = GetDlgItem(of_hwnd, EWFD_PTX_SIZE);
         SetWindowText(cur_ctrl, string_buff);
@@ -1222,44 +1187,67 @@ preview_set_file_info(HWND of_hwnd, gchar *preview_file) {
         return TRUE;
     }
 
-    /* Packets */
-    if(is_breaked) {
-        utf_8to16_snprintf(string_buff, PREVIEW_STR_MAX, "%s, timed out at %u packets",
-            size_str, packet);
+    /* Packet count */
+    if(status == PREVIEW_TIMED_OUT) {
+        utf_8to16_snprintf(string_buff, PREVIEW_STR_MAX, "%s, timed out at %u data records",
+            size_str, stats.data_records);
     } else {
-        utf_8to16_snprintf(string_buff, PREVIEW_STR_MAX, "%s, %u packets",
-            size_str, packet);
+        utf_8to16_snprintf(string_buff, PREVIEW_STR_MAX, "%s, %u data records",
+            size_str, stats.data_records);
     }
     g_free(size_str);
     cur_ctrl = GetDlgItem(of_hwnd, EWFD_PTX_SIZE);
     SetWindowText(cur_ctrl, string_buff);
 
     /* First packet / elapsed time */
-    ti_time = (long)start_time;
-    ti_tm = localtime( &ti_time );
-    if(ti_tm) {
-        StringCchPrintf(first_buff, PREVIEW_STR_MAX,
-                 _T("%04d-%02d-%02d %02d:%02d:%02d"),
-                 ti_tm->tm_year + 1900,
-                 ti_tm->tm_mon + 1,
-                 ti_tm->tm_mday,
-                 ti_tm->tm_hour,
-                 ti_tm->tm_min,
-                 ti_tm->tm_sec);
+    if(stats.have_times) {
+        /*
+         * We saw at least one record with a time stamp, so we can give
+         * a start time (if we have a mix of records with and without
+         * time stamps, and there were records without time stamps
+         * before the one with a time stamp, this may be inaccurate).
+         */
+        ti_time = (long)stats.start_time;
+        ti_tm = localtime( &ti_time );
+        if(ti_tm) {
+            StringCchPrintf(first_buff, PREVIEW_STR_MAX,
+                     _T("%04d-%02d-%02d %02d:%02d:%02d"),
+                     ti_tm->tm_year + 1900,
+                     ti_tm->tm_mon + 1,
+                     ti_tm->tm_mday,
+                     ti_tm->tm_hour,
+                     ti_tm->tm_min,
+                     ti_tm->tm_sec);
+        } else {
+            StringCchPrintf(first_buff, PREVIEW_STR_MAX, _T("?"));
+        }
     } else {
-        StringCchPrintf(first_buff, PREVIEW_STR_MAX, _T("?"));
+        StringCchPrintf(first_buff, PREVIEW_STR_MAX, _T("unknown"));
     }
 
-    elapsed_time = (unsigned int)(stop_time-start_time);
-    if(elapsed_time/86400) {
-        StringCchPrintf(string_buff, PREVIEW_STR_MAX, _T("%s / %02u days %02u:%02u:%02u"),
-        first_buff, elapsed_time/86400, elapsed_time%86400/3600, elapsed_time%3600/60, elapsed_time%60);
+    /* Elapsed time */
+    if(status == PREVIEW_SUCCEEDED && stats.have_times) {
+        /*
+         * We didn't time out, so we looked at all packets, and we got
+         * at least one packet with a time stamp, so we can calculate
+         * an elapsed time from the time stamp of the last packet with
+         * with a time stamp (if we have a mix of records with and without
+         * time stamps, and there were records without time stamps after
+         * the last one with a time stamp, this may be inaccurate).
+         */
+        elapsed_time = (unsigned int)(stats.stop_time-stats.start_time);
+        if(status == PREVIEW_TIMED_OUT) {
+            StringCchPrintf(string_buff, PREVIEW_STR_MAX, _T("%s / unknown"), first_buff);
+        } else if(elapsed_time/86400) {
+            StringCchPrintf(string_buff, PREVIEW_STR_MAX, _T("%s / %02u days %02u:%02u:%02u"),
+                first_buff, elapsed_time/86400, elapsed_time%86400/3600, elapsed_time%3600/60, elapsed_time%60);
+        } else {
+            StringCchPrintf(string_buff, PREVIEW_STR_MAX, _T("%s / %02u:%02u:%02u"),
+                first_buff, elapsed_time%86400/3600, elapsed_time%3600/60, elapsed_time%60);
+        }
     } else {
-        StringCchPrintf(string_buff, PREVIEW_STR_MAX, _T("%s / %02u:%02u:%02u"),
-        first_buff, elapsed_time%86400/3600, elapsed_time%3600/60, elapsed_time%60);
-    }
-    if(is_breaked) {
-        StringCchPrintf(string_buff, PREVIEW_STR_MAX, _T("%s / unknown"), first_buff);
+        StringCchPrintf(string_buff, PREVIEW_STR_MAX, _T("%s / unknown"),
+            first_buff);
     }
     cur_ctrl = GetDlgItem(of_hwnd, EWFD_PTX_START_ELAPSED);
     SetWindowText(cur_ctrl, string_buff);
@@ -1323,8 +1311,7 @@ filter_tb_syntax_check(HWND hwnd, TCHAR *filter_text) {
         SendMessage(hwnd, EM_SETBKGNDCOLOR, (WPARAM) 1, COLOR_WINDOW);
         return;
     } else if (dfilter_compile(utf_16to8(strval), &dfp, NULL)) { /* colorize filter string entry */
-        if (dfp != NULL)
-            dfilter_free(dfp);
+        dfilter_free(dfp);
         /* Valid (light green) */
         SendMessage(hwnd, EM_SETBKGNDCOLOR, 0, RGB(0xe4, 0xff, 0xc7)); /* tango_chameleon_1 */
     } else {
@@ -1332,10 +1319,9 @@ filter_tb_syntax_check(HWND hwnd, TCHAR *filter_text) {
         SendMessage(hwnd, EM_SETBKGNDCOLOR, 0, RGB(0xff, 0xcc, 0xcc)); /* tango_scarlet_red_1 */
     }
 
-    if (strval) g_free(strval);
+    g_free(strval);
 }
 
-
 static UINT_PTR CALLBACK
 open_file_hook_proc(HWND of_hwnd, UINT msg, WPARAM w_param, LPARAM l_param) {
     HWND      cur_ctrl, parent;
@@ -1365,8 +1351,7 @@ open_file_hook_proc(HWND of_hwnd, UINT msg, WPARAM w_param, LPARAM l_param) {
                 case CDN_FILEOK:
                     /* Fetch the read filter */
                     cur_ctrl = GetDlgItem(of_hwnd, EWFD_FILTER_EDIT);
-                    if (g_dfilter_str)
-                        g_free(g_dfilter_str);
+                    g_free(g_dfilter_str);
                     g_dfilter_str = filter_tb_get(cur_ctrl);
 
                     cur_ctrl = GetDlgItem(of_hwnd, EWFD_FORMAT_TYPE);
@@ -1885,7 +1870,6 @@ export_specified_packets_file_hook_proc(HWND sf_hwnd, UINT msg, WPARAM w_param,
     return 0;
 }
 
-
 #define STATIC_LABEL_CHARS 100
 /* For each range static control, fill in its value and enable/disable it. */
 static void
@@ -2220,8 +2204,7 @@ merge_file_hook_proc(HWND mf_hwnd, UINT msg, WPARAM w_param, LPARAM l_param) {
                 case CDN_FILEOK:
                     /* Fetch the read filter */
                     cur_ctrl = GetDlgItem(mf_hwnd, EWFD_FILTER_EDIT);
-                    if (g_dfilter_str)
-                        g_free(g_dfilter_str);
+                    g_free(g_dfilter_str);
                     g_dfilter_str = filter_tb_get(cur_ctrl);
 
                     cur_ctrl = GetDlgItem(mf_hwnd, EWFD_MERGE_CHRONO_BTN);
@@ -2340,7 +2323,7 @@ export_raw_file_hook_proc(HWND ef_hwnd, UINT msg, WPARAM w_param, LPARAM l_param
     switch(msg) {
         case WM_INITDIALOG:
             StringCchPrintf(raw_msg, STATIC_LABEL_CHARS, _T("%d byte%s of raw binary data will be written"),
-                    ofnp->lCustData, utf_8to16(plurality(ofnp->lCustData, "", "s")));
+                    (int)ofnp->lCustData, utf_8to16(plurality(ofnp->lCustData, "", "s")));
             cur_ctrl = GetDlgItem(ef_hwnd, EWFD_EXPORTRAW_ST);
             SetWindowText(cur_ctrl, raw_msg);
             break;
@@ -2368,7 +2351,7 @@ export_sslkeys_file_hook_proc(HWND ef_hwnd, UINT msg, WPARAM w_param, LPARAM l_p
     switch(msg) {
         case WM_INITDIALOG:
             StringCchPrintf(sslkeys_msg, STATIC_LABEL_CHARS, _T("%d SSL Session Key%s will be written"),
-                    ofnp->lCustData, utf_8to16(plurality(ofnp->lCustData, "", "s")));
+                    (int)ofnp->lCustData, utf_8to16(plurality(ofnp->lCustData, "", "s")));
             cur_ctrl = GetDlgItem(ef_hwnd, EWFD_EXPORTSSLKEYS_ST);
             SetWindowText(cur_ctrl, sslkeys_msg);
             break;
